@@ -22,6 +22,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import org.aspectj.ajdt.internal.compiler.CommonPrinter;
 import org.aspectj.ajdt.internal.compiler.ast.AspectDeclaration;
 import org.aspectj.ajdt.internal.compiler.ast.PointcutDeclaration;
 import org.aspectj.asm.AsmManager;
@@ -61,6 +62,7 @@ import org.aspectj.weaver.ReferenceType;
 import org.aspectj.weaver.ReferenceTypeDelegate;
 import org.aspectj.weaver.ResolvedMember;
 import org.aspectj.weaver.ResolvedType;
+import org.aspectj.weaver.ResolvedTypeMunger;
 import org.aspectj.weaver.UnresolvedType;
 import org.aspectj.weaver.WeaverMessages;
 import org.aspectj.weaver.WeaverStateInfo;
@@ -87,7 +89,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 	public EclipseFactory factory = null;
 
 	// private boolean builtInterTypesAndPerClauses = false;
-	private final List pendingTypesToWeave = new ArrayList();
+	private final List<SourceTypeBinding> pendingTypesToWeave = new ArrayList<SourceTypeBinding>();
 
 	// Q: What are dangerousInterfaces?
 	// A: An interface is considered dangerous if an ITD has been made upon it
@@ -174,28 +176,39 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		// need to build inter-type declarations for all AspectDeclarations at
 		// this point
 		// this MUST be done in order from super-types to subtypes
-		List typesToProcess = new ArrayList();
+		List<SourceTypeBinding> typesToProcess = new ArrayList<SourceTypeBinding>();
+		List<SourceTypeBinding> aspectsToProcess = new ArrayList<SourceTypeBinding>();
 		for (int i = lastCompletedUnitIndex + 1; i <= lastUnitIndex; i++) {
 			CompilationUnitScope cus = units[i].scope;
 			SourceTypeBinding[] stbs = cus.topLevelTypes;
 			for (int j = 0; j < stbs.length; j++) {
 				SourceTypeBinding stb = stbs[j];
 				typesToProcess.add(stb);
+				TypeDeclaration typeDeclaration = stb.scope.referenceContext;
+				if (typeDeclaration instanceof AspectDeclaration) {
+					aspectsToProcess.add(stb);
+				}
 			}
 		}
 		factory.getWorld().getCrosscuttingMembersSet().reset();
+
+		// Need to do these before the other ITDs
+		for (SourceTypeBinding aspectToProcess : aspectsToProcess) {
+			processInterTypeMemberTypes(aspectToProcess.scope);
+		}
+
 		while (typesToProcess.size() > 0) {
 			// removes types from the list as they are processed...
-			collectAllITDsAndDeclares((SourceTypeBinding) typesToProcess.get(0), typesToProcess);
+			collectAllITDsAndDeclares(typesToProcess.get(0), typesToProcess);
 		}
 
 		factory.finishTypeMungers();
 
 		// now do weaving
-		Collection typeMungers = factory.getTypeMungers();
+		List<ConcreteTypeMunger> typeMungers = factory.getTypeMungers();
 
-		Collection declareParents = factory.getDeclareParents();
-		Collection declareAnnotationOnTypes = factory.getDeclareAnnotationOnTypes();
+		List<DeclareParents> declareParents = factory.getDeclareParents();
+		List<DeclareAnnotation> declareAnnotationOnTypes = factory.getDeclareAnnotationOnTypes();
 
 		doPendingWeaves();
 
@@ -237,15 +250,12 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 			while (typesToProcess.size() > 0) {
 				// A side effect of weaveIntertypes() is that the processed type
 				// is removed from the collection
-				weaveIntertypes(typesToProcess, (SourceTypeBinding) typesToProcess.get(0), typeMungers, declareParents,
-						declareAnnotationOnTypes);
+				weaveIntertypes(typesToProcess, typesToProcess.get(0), typeMungers, declareParents, declareAnnotationOnTypes);
 			}
 
 		} else {
 			// Order isn't important
 			for (int i = lastCompletedUnitIndex + 1; i <= lastUnitIndex; i++) {
-				// System.err.println("Working on "+new
-				// String(units[i].getFileName()));
 				weaveInterTypeDeclarations(units[i].scope, typeMungers, declareParents, declareAnnotationOnTypes);
 			}
 		}
@@ -357,8 +367,9 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		ReferenceBinding[] memberTypes = sourceType.memberTypes;
 		for (int i = 0, length = memberTypes.length; i < length; i++) {
 			SourceTypeBinding rb = (SourceTypeBinding) memberTypes[i];
-			if (!rb.superclass().equals(sourceType))
+			if (!rb.superclass().equals(sourceType)) {
 				doSupertypesFirst(rb.superclass(), yetToProcess);
+			}
 		}
 
 		buildInterTypeAndPerClause(sourceType.scope);
@@ -373,8 +384,9 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 	 * problems if you supply 'pieces' of a hierarchy, i.e. the bottom and the top, but not the middle - but what the hell are you
 	 * doing if you do that?
 	 */
-	private void weaveIntertypes(List typesToProcess, SourceTypeBinding typeToWeave, Collection typeMungers,
-			Collection declareParents, Collection declareAnnotationOnTypes) {
+	private void weaveIntertypes(List<SourceTypeBinding> typesToProcess, SourceTypeBinding typeToWeave,
+			List<ConcreteTypeMunger> typeMungers, List<DeclareParents> declareParents,
+			List<DeclareAnnotation> declareAnnotationOnTypes) {
 		// Look at the supertype first
 		ReferenceBinding superType = typeToWeave.superclass();
 		if (typesToProcess.contains(superType) && superType instanceof SourceTypeBinding) {
@@ -483,13 +495,26 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		boolean couldBeAtAspect = false;
 		if (annotations != null) {
 			for (int i = 0; i < annotations.length && !couldBeAtAspect; i++) {
-				if (annotations[i].toString().equals("@Aspect"))
+				if (annotations[i].toString().equals("@Aspect")) {
 					couldBeAtAspect = true;
+				}
 			}
 		}
 		return couldBeAtAspect;
 	}
 
+	/**
+	 * Applies any intertype member type declarations up front.
+	 */
+	private void processInterTypeMemberTypes(ClassScope classScope) {
+		TypeDeclaration dec = classScope.referenceContext;
+		if (dec instanceof AspectDeclaration) {
+			((AspectDeclaration) dec).processIntertypeMemberTypes(classScope);
+		}
+		// if we are going to support nested aspects making itd member types, copy the logic from the end of
+		// buildInterTypeAndPerClause() which walks members
+	}
+
 	private void buildInterTypeAndPerClause(ClassScope s) {
 		TypeDeclaration dec = s.referenceContext;
 		if (dec instanceof AspectDeclaration) {
@@ -521,22 +546,29 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 			for (int i = 0; i < decl.annotations.length; i++) {
 				Annotation ann = decl.annotations[i];
 				if (ann.type instanceof SingleTypeReference) {
-					if (CharOperation.equals("Aspect".toCharArray(), ((SingleTypeReference) ann.type).token))
+					if (CharOperation.equals("Aspect".toCharArray(), ((SingleTypeReference) ann.type).token)) {
 						return true;
+					}
 				} else if (ann.type instanceof QualifiedTypeReference) {
 					QualifiedTypeReference qtr = (QualifiedTypeReference) ann.type;
-					if (qtr.tokens.length != 5)
+					if (qtr.tokens.length != 5) {
 						return false;
-					if (!CharOperation.equals("org".toCharArray(), qtr.tokens[0]))
+					}
+					if (!CharOperation.equals("org".toCharArray(), qtr.tokens[0])) {
 						return false;
-					if (!CharOperation.equals("aspectj".toCharArray(), qtr.tokens[1]))
+					}
+					if (!CharOperation.equals("aspectj".toCharArray(), qtr.tokens[1])) {
 						return false;
-					if (!CharOperation.equals("lang".toCharArray(), qtr.tokens[2]))
+					}
+					if (!CharOperation.equals("lang".toCharArray(), qtr.tokens[2])) {
 						return false;
-					if (!CharOperation.equals("annotation".toCharArray(), qtr.tokens[3]))
+					}
+					if (!CharOperation.equals("annotation".toCharArray(), qtr.tokens[3])) {
 						return false;
-					if (!CharOperation.equals("Aspect".toCharArray(), qtr.tokens[4]))
+					}
+					if (!CharOperation.equals("Aspect".toCharArray(), qtr.tokens[4])) {
 						return false;
+					}
 					return true;
 				}
 			}
@@ -544,8 +576,8 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		return false;
 	}
 
-	private void weaveInterTypeDeclarations(CompilationUnitScope unit, Collection typeMungers, Collection declareParents,
-			Collection declareAnnotationOnTypes) {
+	private void weaveInterTypeDeclarations(CompilationUnitScope unit, List<ConcreteTypeMunger> typeMungers,
+			List<DeclareParents> declareParents, List<DeclareAnnotation> declareAnnotationOnTypes) {
 		for (int i = 0, length = unit.topLevelTypes.length; i < length; i++) {
 			weaveInterTypeDeclarations(unit.topLevelTypes[i], typeMungers, declareParents, declareAnnotationOnTypes, false);
 		}
@@ -553,16 +585,61 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 
 	private void weaveInterTypeDeclarations(SourceTypeBinding sourceType) {
 		if (!factory.areTypeMungersFinished()) {
-			if (!pendingTypesToWeave.contains(sourceType))
+			if (!pendingTypesToWeave.contains(sourceType)) {
 				pendingTypesToWeave.add(sourceType);
+
+// inner type ITD support - may need this for some incremental cases...
+				// List<ConcreteTypeMunger> ctms = factory.getWorld().getCrosscuttingMembersSet().getTypeMungersOfKind(
+				// ResolvedTypeMunger.InnerClass);
+				// // List<ConcreteTypeMunger> innerTypeMungers = new ArrayList<ConcreteTypeMunger>();
+				// // for (ConcreteTypeMunger ctm : ctms) {
+				// // if (ctm.getMunger() != null && ctm.getMunger().getKind() == ResolvedTypeMunger.InnerClass) {
+				// // innerTypeMungers.add(ctm);
+				// // }
+				// // }
+				// // that includes the innertype one...
+				// // doPendingWeaves at this level is about applying inner class
+				// BinaryTypeBinding t = (BinaryTypeBinding) sourceType;
+				// for (ConcreteTypeMunger ctm : innerTypeMungers) {
+				// NewMemberClassTypeMunger nmctm = (NewMemberClassTypeMunger) ctm.getMunger();
+				// ReferenceBinding[] rbs = t.memberTypes;
+				// UnresolvedType ut = factory.fromBinding(t);
+				// if (ut.equals(nmctm.getTargetType())) {
+				// // got a match here
+				// SourceTypeBinding aspectTypeBinding = (SourceTypeBinding) factory.makeTypeBinding(ctm.getAspectType());
+				//
+				// char[] mungerMemberTypeName = ("$" + nmctm.getMemberTypeName()).toCharArray();
+				// ReferenceBinding innerTypeBinding = null;
+				// for (ReferenceBinding innerType : aspectTypeBinding.memberTypes) {
+				// char[] compounded = CharOperation.concatWith(innerType.compoundName, '.');
+				// if (org.aspectj.org.eclipse.jdt.core.compiler.CharOperation.endsWith(compounded, mungerMemberTypeName)) {
+				// innerTypeBinding = innerType;
+				// break;
+				// }
+				// }
+				// // may be unresolved if the aspect type binding was a BinaryTypeBinding
+				// if (innerTypeBinding instanceof UnresolvedReferenceBinding) {
+				// innerTypeBinding = BinaryTypeBinding
+				// .resolveType(innerTypeBinding, factory.getLookupEnvironment(), true);
+				// }
+				// t.memberTypes(); // cause initialization
+				// t.memberTypes = new ReferenceBinding[] { innerTypeBinding };
+				//
+				// int stop = 1;
+				// // The inner type from the aspect should be put into the membertypebindings for this
+				//
+				// }
+				// }
+
+			}
 		} else {
 			weaveInterTypeDeclarations(sourceType, factory.getTypeMungers(), factory.getDeclareParents(), factory
 					.getDeclareAnnotationOnTypes(), true);
 		}
 	}
 
-	private void weaveInterTypeDeclarations(SourceTypeBinding sourceType, Collection typeMungers, Collection declareParents,
-			Collection declareAnnotationOnTypes, boolean skipInners) {
+	private void weaveInterTypeDeclarations(SourceTypeBinding sourceType, List<ConcreteTypeMunger> typeMungers,
+			List<DeclareParents> declareParents, List<DeclareAnnotation> declareAnnotationOnTypes, boolean skipInners) {
 
 		ContextToken tok = CompilationAndWeavingContext.enteringPhase(CompilationAndWeavingContext.WEAVING_INTERTYPE_DECLARATIONS,
 				sourceType.sourceName);
@@ -570,8 +647,9 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		ResolvedType onType = factory.fromEclipse(sourceType);
 
 		// AMC we shouldn't need this when generic sigs are fixed??
-		if (onType.isRawType())
+		if (onType.isRawType()) {
 			onType = onType.getGenericType();
+		}
 
 		WeaverStateInfo info = onType.getWeaverState();
 
@@ -606,6 +684,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		boolean needOldStyleWarning = (info != null && info.isOldStyle());
 
 		onType.clearInterTypeMungers();
+		onType.ensureConsistent();
 
 		// FIXME asc perf Could optimize here, after processing the expected set
 		// of types we may bring
@@ -616,7 +695,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		// messages...
 
 		List decpToRepeat = new ArrayList();
-		List decaToRepeat = new ArrayList();
+		List<DeclareAnnotation> decaToRepeat = new ArrayList<DeclareAnnotation>();
 		boolean anyNewParents = false;
 		boolean anyNewAnnotations = false;
 
@@ -631,22 +710,27 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 			if (!decp.isMixin()) {
 				boolean didSomething = doDeclareParents(decp, sourceType);
 				if (didSomething) {
+					if (factory.pushinCollector != null) {
+						factory.pushinCollector.tagAsMunged(sourceType, decp.getParents().get(0));
+					}
 					anyNewParents = true;
 				} else {
-					if (!decp.getChild().isStarAnnotation())
+					if (!decp.getChild().isStarAnnotation()) {
 						decpToRepeat.add(decp);
+					}
 				}
 			}
 		}
 
-		for (Iterator i = declareAnnotationOnTypes.iterator(); i.hasNext();) {
-			DeclareAnnotation deca = (DeclareAnnotation) i.next();
+		for (Iterator<DeclareAnnotation> i = declareAnnotationOnTypes.iterator(); i.hasNext();) {
+			DeclareAnnotation deca = i.next();
 			boolean didSomething = doDeclareAnnotations(deca, sourceType, true);
 			if (didSomething) {
 				anyNewAnnotations = true;
 			} else {
-				if (!deca.getTypePattern().isStar())
+				if (!deca.getTypePattern().isStar()) {
 					decaToRepeat.add(deca);
+				}
 			}
 		}
 
@@ -658,17 +742,23 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 				DeclareParents decp = (DeclareParents) i.next();
 				boolean didSomething = doDeclareParents(decp, sourceType);
 				if (didSomething) {
+					if (factory.pushinCollector != null) {
+						factory.pushinCollector.tagAsMunged(sourceType, decp.getParents().get(0));
+					}
 					anyNewParents = true;
 					forRemoval.add(decp);
 				}
 			}
 			decpToRepeat.removeAll(forRemoval);
 
-			forRemoval = new ArrayList();
+			forRemoval.clear();
 			for (Iterator i = declareAnnotationOnTypes.iterator(); i.hasNext();) {
 				DeclareAnnotation deca = (DeclareAnnotation) i.next();
 				boolean didSomething = doDeclareAnnotations(deca, sourceType, false);
 				if (didSomething) {
+					if (factory.pushinCollector != null) {
+						factory.pushinCollector.tagAsMunged(sourceType, deca.getAnnotationString());
+					}
 					anyNewAnnotations = true;
 					forRemoval.add(deca);
 				}
@@ -676,7 +766,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 			decaToRepeat.removeAll(forRemoval);
 		}
 
-		for (Iterator i = typeMungers.iterator(); i.hasNext();) {
+		for (Iterator<ConcreteTypeMunger> i = typeMungers.iterator(); i.hasNext();) {
 			EclipseTypeMunger munger = (EclipseTypeMunger) i.next();
 			if (munger.matches(onType)) {
 				if (needOldStyleWarning) {
@@ -684,16 +774,30 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 							+ " should be recompiled with ajc-1.1.1 for best results", onType.getSourceLocation(), null);
 					needOldStyleWarning = false;
 				}
-				onType.addInterTypeMunger(munger);
+				onType.addInterTypeMunger(munger, true);
+				if (munger.getMunger()!=null && munger.getMunger().getKind() == ResolvedTypeMunger.InnerClass) {
+					// Must do these right now, because if we do an ITD member afterwards it may attempt to reference the
+					// type being applied (the call above 'addInterTypeMunger' will fail for these ITDs if it needed
+					// it to be in place)
+					if (munger.munge(sourceType, onType)) {
+						if (factory.pushinCollector != null) {
+							factory.pushinCollector.tagAsMunged(sourceType, munger.getSourceMethod());
+						}
+					}
+				}
 			}
 		}
 
 		onType.checkInterTypeMungers();
 		for (Iterator i = onType.getInterTypeMungers().iterator(); i.hasNext();) {
 			EclipseTypeMunger munger = (EclipseTypeMunger) i.next();
-			// System.out.println("applying: " + munger + " to " + new
-			// String(sourceType.sourceName));
-			munger.munge(sourceType, onType);
+			if (munger.getMunger()==null || munger.getMunger().getKind() != ResolvedTypeMunger.InnerClass) {
+				if (munger.munge(sourceType, onType)) {
+					if (factory.pushinCollector != null) {
+						factory.pushinCollector.tagAsMunged(sourceType, munger.getSourceMethod());
+					}
+				}
+			}
 		}
 
 		// Call if you would like to do source weaving of declare
@@ -769,8 +873,9 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 				} else {
 					// do not actually do it if the type isn't exposed - this
 					// will correctly reported as a problem elsewhere
-					if (!resolvedSourceType.isExposedToWeaver())
+					if (!resolvedSourceType.isExposedToWeaver()) {
 						return false;
+					}
 					// AsmRelationshipProvider.getDefault().
 					// addDeclareParentsRelationship
 					// (declareParents.getSourceLocation(),
@@ -786,32 +891,42 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 	}
 
 	private String stringifyTargets(long bits) {
-		if ((bits & TagBits.AnnotationTargetMASK) == 0)
+		if ((bits & TagBits.AnnotationTargetMASK) == 0) {
 			return "";
+		}
 		Set s = new HashSet();
-		if ((bits & TagBits.AnnotationForAnnotationType) != 0)
+		if ((bits & TagBits.AnnotationForAnnotationType) != 0) {
 			s.add("ANNOTATION_TYPE");
-		if ((bits & TagBits.AnnotationForConstructor) != 0)
+		}
+		if ((bits & TagBits.AnnotationForConstructor) != 0) {
 			s.add("CONSTRUCTOR");
-		if ((bits & TagBits.AnnotationForField) != 0)
+		}
+		if ((bits & TagBits.AnnotationForField) != 0) {
 			s.add("FIELD");
-		if ((bits & TagBits.AnnotationForLocalVariable) != 0)
+		}
+		if ((bits & TagBits.AnnotationForLocalVariable) != 0) {
 			s.add("LOCAL_VARIABLE");
-		if ((bits & TagBits.AnnotationForMethod) != 0)
+		}
+		if ((bits & TagBits.AnnotationForMethod) != 0) {
 			s.add("METHOD");
-		if ((bits & TagBits.AnnotationForPackage) != 0)
+		}
+		if ((bits & TagBits.AnnotationForPackage) != 0) {
 			s.add("PACKAGE");
-		if ((bits & TagBits.AnnotationForParameter) != 0)
+		}
+		if ((bits & TagBits.AnnotationForParameter) != 0) {
 			s.add("PARAMETER");
-		if ((bits & TagBits.AnnotationForType) != 0)
+		}
+		if ((bits & TagBits.AnnotationForType) != 0) {
 			s.add("TYPE");
+		}
 		StringBuffer sb = new StringBuffer();
 		sb.append("{");
 		for (Iterator iter = s.iterator(); iter.hasNext();) {
 			String element = (String) iter.next();
 			sb.append(element);
-			if (iter.hasNext())
+			if (iter.hasNext()) {
 				sb.append(",");
+			}
 		}
 		sb.append("}");
 		return sb.toString();
@@ -819,10 +934,12 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 
 	private boolean doDeclareAnnotations(DeclareAnnotation decA, SourceTypeBinding sourceType, boolean reportProblems) {
 		ResolvedType rtx = factory.fromEclipse(sourceType);
-		if (!decA.matches(rtx))
+		if (!decA.matches(rtx)) {
 			return false;
-		if (!rtx.isExposedToWeaver())
+		}
+		if (!rtx.isExposedToWeaver()) {
 			return false;
+		}
 
 		ContextToken tok = CompilationAndWeavingContext.enteringPhase(CompilationAndWeavingContext.PROCESSING_DECLARE_ANNOTATIONS,
 				sourceType.sourceName);
@@ -859,6 +976,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		Annotation[] toAdd = null;
 		long abits = 0;
 
+		AbstractMethodDeclaration methodDecl = null;
 		// Might have to retrieve the annotation through BCEL and construct an
 		// eclipse one for it.
 		if (stb instanceof BinaryTypeBinding) {
@@ -882,8 +1000,9 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 					// What does that mean? It means on an incremental compile you
 					// might get away with an
 					// annotation that isn't allowed on a type being put on a type.
-					if (toAdd[0].resolvedType != null) // pr184447
+					if (toAdd[0].resolvedType != null) {
 						abits = toAdd[0].resolvedType.getAnnotationTagBits();
+					}
 				}
 			}
 		} else if (stb != null) {
@@ -893,12 +1012,13 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 			MethodBinding[] mbs = stb.getMethods(decA.getAnnotationMethod().toCharArray());
 			abits = mbs[0].getAnnotationTagBits(); // ensure resolved
 			TypeDeclaration typeDecl = ((SourceTypeBinding) mbs[0].declaringClass).scope.referenceContext;
-			AbstractMethodDeclaration methodDecl = typeDecl.declarationOf(mbs[0]);
+			methodDecl = typeDecl.declarationOf(mbs[0]);
 			toAdd = methodDecl.annotations; // this is what to add
 			toAdd[0] = createAnnotationCopy(toAdd[0]);
-			if (toAdd[0].resolvedType != null) // pr148536
+			if (toAdd[0].resolvedType != null) {
 				abits = toAdd[0].resolvedType.getAnnotationTagBits();
-			// }
+				// }
+			}
 		}
 
 		// This happens if there is another error in the code - that should be reported separately
@@ -912,6 +1032,11 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 			// on the weaver type temporarily
 			ResolvedType theTargetType = factory.fromEclipse(sourceType);
 			TypeBinding theAnnotationType = toAdd[0].resolvedType;
+			// The annotation type may be null if it could not be resolved (eg. the relevant import has not been added yet)
+			// In this case an error will be put out about the annotation but not if we crash here
+			if (theAnnotationType == null) {
+				return false;
+			}
 			String sig = new String(theAnnotationType.signature());
 			UnresolvedType bcelAnnotationType = UnresolvedType.forSignature(sig);
 			String name = bcelAnnotationType.getName();
@@ -964,7 +1089,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		}
 
 		Annotation currentAnnotations[] = sourceType.scope.referenceContext.annotations;
-		if (currentAnnotations != null)
+		if (currentAnnotations != null) {
 			for (int i = 0; i < currentAnnotations.length; i++) {
 				Annotation annotation = currentAnnotations[i];
 				String a = CharOperation.toString(annotation.type.getTypeName());
@@ -979,6 +1104,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 					return false;
 				}
 			}
+		}
 
 		if (((abits & TagBits.AnnotationTargetMASK) != 0)) {
 			if ((abits & (TagBits.AnnotationForAnnotationType | TagBits.AnnotationForType)) == 0) {
@@ -1030,6 +1156,10 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		}
 		sourceType.scope.referenceContext.annotations = newset;
 		CompilationAndWeavingContext.leavingPhase(tok);
+		if (factory.pushinCollector != null) {
+			factory.pushinCollector.tagAsMunged(sourceType, new CommonPrinter((methodDecl == null ? null : methodDecl.scope))
+					.printAnnotation(toAdd[0]).toString());
+		}
 		return true;
 	}
 
@@ -1045,8 +1175,9 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		// String theName = annX.getSignature().getBaseName();
 		char[][] typeName = CharOperation.splitOn('.', name.replace('$', '.').toCharArray()); // pr149293 - not bulletproof...
 		long[] positions = new long[typeName.length];
-		for (int i = 0; i < positions.length; i++)
+		for (int i = 0; i < positions.length; i++) {
 			positions[i] = pos;
+		}
 		TypeReference annType = new QualifiedTypeReference(typeName, positions);
 		NormalAnnotation ann = new NormalAnnotation(annType, pos);
 		ann.resolvedType = tb; // yuck - is this OK in all cases?
@@ -1108,8 +1239,9 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 			String filename = new String(sourceType.getFileName());
 
 			int takefrom = filename.lastIndexOf('/');
-			if (takefrom == -1)
+			if (takefrom == -1) {
 				takefrom = filename.lastIndexOf('\\');
+			}
 			filename = filename.substring(takefrom + 1);
 
 			factory.getWorld().getMessageHandler().handleMessage(
@@ -1128,8 +1260,9 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 
 	private void addParent(SourceTypeBinding sourceType, ResolvedType parent) {
 		ReferenceBinding parentBinding = (ReferenceBinding) factory.makeTypeBinding(parent);
-		if (parentBinding == null)
+		if (parentBinding == null) {
 			return; // The parent is missing, it will be reported elsewhere.
+		}
 		sourceType.rememberTypeHierarchy();
 		if (parentBinding.isClass()) {
 			sourceType.superclass = parentBinding;
