@@ -286,14 +286,15 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 	public boolean visit(TypeDeclaration typeDeclaration, CompilationUnitScope scope) {
 		String name = new String(typeDeclaration.name);
 		IProgramElement.Kind kind = IProgramElement.Kind.CLASS;
-		if (typeDeclaration instanceof AspectDeclaration)
+		if (typeDeclaration instanceof AspectDeclaration) {
 			kind = IProgramElement.Kind.ASPECT;
-		else if (TypeDeclaration.kind(typeDeclaration.modifiers) == TypeDeclaration.INTERFACE_DECL)
+		} else if (TypeDeclaration.kind(typeDeclaration.modifiers) == TypeDeclaration.INTERFACE_DECL) {
 			kind = IProgramElement.Kind.INTERFACE;
-		else if (TypeDeclaration.kind(typeDeclaration.modifiers) == TypeDeclaration.ENUM_DECL)
+		} else if (TypeDeclaration.kind(typeDeclaration.modifiers) == TypeDeclaration.ENUM_DECL) {
 			kind = IProgramElement.Kind.ENUM;
-		else if (TypeDeclaration.kind(typeDeclaration.modifiers) == TypeDeclaration.ANNOTATION_TYPE_DECL)
+		} else if (TypeDeclaration.kind(typeDeclaration.modifiers) == TypeDeclaration.ANNOTATION_TYPE_DECL) {
 			kind = IProgramElement.Kind.ANNOTATION;
+		}
 
 		boolean isAnnotationStyleAspect = false;
 		// @AJ support
@@ -381,14 +382,15 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 
 		IProgramElement.Kind kind = IProgramElement.Kind.CLASS;
 		int typeDeclarationKind = TypeDeclaration.kind(memberTypeDeclaration.modifiers);
-		if (memberTypeDeclaration instanceof AspectDeclaration)
+		if (memberTypeDeclaration instanceof AspectDeclaration) {
 			kind = IProgramElement.Kind.ASPECT;
-		else if (typeDeclarationKind == TypeDeclaration.INTERFACE_DECL)
+		} else if (typeDeclarationKind == TypeDeclaration.INTERFACE_DECL) {
 			kind = IProgramElement.Kind.INTERFACE;
-		else if (typeDeclarationKind == TypeDeclaration.ENUM_DECL)
+		} else if (typeDeclarationKind == TypeDeclaration.ENUM_DECL) {
 			kind = IProgramElement.Kind.ENUM;
-		else if (typeDeclarationKind == TypeDeclaration.ANNOTATION_TYPE_DECL)
+		} else if (typeDeclarationKind == TypeDeclaration.ANNOTATION_TYPE_DECL) {
 			kind = IProgramElement.Kind.ANNOTATION;
+		}
 
 		boolean isAnnotationStyleAspect = false;
 		// @AJ support
@@ -442,12 +444,13 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 		}
 
 		IProgramElement.Kind kind = IProgramElement.Kind.CLASS;
-		if (TypeDeclaration.kind(memberTypeDeclaration.modifiers) == TypeDeclaration.INTERFACE_DECL)
+		if (TypeDeclaration.kind(memberTypeDeclaration.modifiers) == TypeDeclaration.INTERFACE_DECL) {
 			kind = IProgramElement.Kind.INTERFACE;
-		else if (TypeDeclaration.kind(memberTypeDeclaration.modifiers) == TypeDeclaration.ENUM_DECL)
+		} else if (TypeDeclaration.kind(memberTypeDeclaration.modifiers) == TypeDeclaration.ENUM_DECL) {
 			kind = IProgramElement.Kind.ENUM;
-		else if (TypeDeclaration.kind(memberTypeDeclaration.modifiers) == TypeDeclaration.ANNOTATION_TYPE_DECL)
+		} else if (TypeDeclaration.kind(memberTypeDeclaration.modifiers) == TypeDeclaration.ANNOTATION_TYPE_DECL) {
 			kind = IProgramElement.Kind.ANNOTATION;
+		}
 
 		// @AJ support
 		boolean isAnnotationStyleAspect = false;
@@ -545,7 +548,15 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 			}
 			if (decl instanceof DeclareAnnotation) {
 				ResolvedType annotationType = ((DeclareAnnotation) decl).getAnnotationType();
-				peNode.setAnnotationType(annotationType.getName());
+				if (annotationType == null) {
+					String s = ((DeclareAnnotation) decl).getAnnotationString();
+					if (s != null && s.length() > 0) {
+						s = s.substring(1);
+					}
+					peNode.setAnnotationType(s);
+				} else {
+					peNode.setAnnotationType(annotationType.getName());
+				}
 			}
 		}
 		if (methodDeclaration.returnType != null) {
@@ -557,15 +568,17 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 				InterTypeFieldDeclaration itfd = (InterTypeFieldDeclaration) methodDeclaration;
 				if (itfd.getRealFieldType() != null) {
 					peNode.setCorrespondingType(new String(itfd.getRealFieldType().readableName()));
-				} else
+				} else {
 					peNode.setCorrespondingType(null);
+				}
 
 				// was peNode.setCorrespondingType(methodDeclaration.returnType.toString());
 			} else {
-				if (methodDeclaration.returnType.resolvedType != null)
+				if (methodDeclaration.returnType.resolvedType != null) {
 					peNode.setCorrespondingType(new String(methodDeclaration.returnType.resolvedType.readableName()));
-				else
+				} else {
 					peNode.setCorrespondingType(null);
+				}
 			}
 		} else {
 			peNode.setCorrespondingType(null);
@@ -637,11 +650,13 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 	private List genNamedPointcuts(MethodDeclaration methodDeclaration) {
 		List pointcuts = new ArrayList();
 		if (methodDeclaration instanceof AdviceDeclaration) {
-			if (((AdviceDeclaration) methodDeclaration).pointcutDesignator != null)
+			if (((AdviceDeclaration) methodDeclaration).pointcutDesignator != null) {
 				addAllNamed(((AdviceDeclaration) methodDeclaration).pointcutDesignator.getPointcut(), pointcuts);
+			}
 		} else if (methodDeclaration instanceof PointcutDeclaration) {
-			if (((PointcutDeclaration) methodDeclaration).pointcutDesignator != null)
+			if (((PointcutDeclaration) methodDeclaration).pointcutDesignator != null) {
 				addAllNamed(((PointcutDeclaration) methodDeclaration).pointcutDesignator.getPointcut(), pointcuts);
+			}
 		}
 		return pointcuts;
 	}
@@ -651,8 +666,9 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 	 * @param pointcuts accumulator for named pointcuts
 	 */
 	private void addAllNamed(Pointcut pointcut, List pointcuts) {
-		if (pointcut == null)
+		if (pointcut == null) {
 			return;
+		}
 		if (pointcut instanceof ReferencePointcut) {
 			ReferencePointcut rp = (ReferencePointcut) pointcut;
 			pointcuts.add(rp);
@@ -687,8 +703,9 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 		methodDeclaration.printReturnType(0, output).append(methodDeclaration.selector).append('(');
 		if (methodDeclaration.arguments != null) {
 			for (int i = 0; i < methodDeclaration.arguments.length; i++) {
-				if (i > 0)
+				if (i > 0) {
 					output.append(", "); //$NON-NLS-1$
+				}
 				methodDeclaration.arguments[i].print(0, output);
 			}
 		}
@@ -696,8 +713,9 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 		if (methodDeclaration.thrownExceptions != null) {
 			output.append(" throws "); //$NON-NLS-1$
 			for (int i = 0; i < methodDeclaration.thrownExceptions.length; i++) {
-				if (i > 0)
+				if (i > 0) {
 					output.append(", "); //$NON-NLS-1$
+				}
 				methodDeclaration.thrownExceptions[i].print(0, output);
 			}
 		}
@@ -830,8 +848,9 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 	 * Checks if comments should be added to the model before generating.
 	 */
 	protected String generateJavadocComment(ASTNode astNode) {
-		if (buildConfig != null && !buildConfig.isGenerateJavadocsInModelMode())
+		if (buildConfig != null && !buildConfig.isGenerateJavadocsInModelMode()) {
 			return null;
+		}
 
 		// StringBuffer sb = new StringBuffer(); // !!! specify length?
 		// boolean completed = false;
@@ -853,8 +872,9 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 					astNode.sourceStart);
 			while (comment.length > 2) {
 				int star = CharOperation.indexOf('*', comment);
-				if (star == -1)
+				if (star == -1) {
 					return null;
+				}
 				// looking for '/**' and not '//' or '//*'
 				if (star != 0 && (comment[star - 1] == '/') && (comment[star + 1] == '*')
 						&& (star - 2 < 0 || comment[star - 2] != '/')) {
@@ -935,8 +955,9 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 		if (constructorDeclaration.arguments != null) {
 			for (int i = 0; i < constructorDeclaration.arguments.length; i++) {
 				argumentsSignature.append(constructorDeclaration.arguments[i].type);
-				if (i + 1 < constructorDeclaration.arguments.length)
+				if (i + 1 < constructorDeclaration.arguments.length) {
 					argumentsSignature.append(",");
+				}
 			}
 		}
 		argumentsSignature.append(")");
@@ -995,8 +1016,9 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 		output.append(constructorDeclaration.selector).append('(');
 		if (constructorDeclaration.arguments != null) {
 			for (int i = 0; i < constructorDeclaration.arguments.length; i++) {
-				if (i > 0)
+				if (i > 0) {
 					output.append(", "); //$NON-NLS-1$
+				}
 				constructorDeclaration.arguments[i].print(0, output);
 			}
 		}
@@ -1004,8 +1026,9 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 		if (constructorDeclaration.thrownExceptions != null) {
 			output.append(" throws "); //$NON-NLS-1$
 			for (int i = 0; i < constructorDeclaration.thrownExceptions.length; i++) {
-				if (i > 0)
+				if (i > 0) {
 					output.append(", "); //$NON-NLS-1$
+				}
 				constructorDeclaration.thrownExceptions[i].print(0, output);
 			}
 		}
@@ -1034,8 +1057,9 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 	private Initializer inInitializer = null;
 
 	public boolean visit(Initializer initializer, MethodScope scope) {
-		if (initializer == inInitializer)
+		if (initializer == inInitializer) {
 			return false;
+		}
 		inInitializer = initializer;
 
 		IProgramElement peNode = new ProgramElement(activeStructureModel, "...", IProgramElement.Kind.INITIALIZER,
@@ -1091,12 +1115,15 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 	// various ASTNode types. They have no common ancestor in the
 	// hierarchy!!
 	protected int getEndLine(ASTNode n) {
-		if (n instanceof AbstractVariableDeclaration)
+		if (n instanceof AbstractVariableDeclaration) {
 			return getEndLine((AbstractVariableDeclaration) n);
-		if (n instanceof AbstractMethodDeclaration)
+		}
+		if (n instanceof AbstractMethodDeclaration) {
 			return getEndLine((AbstractMethodDeclaration) n);
-		if (n instanceof TypeDeclaration)
+		}
+		if (n instanceof TypeDeclaration) {
 			return getEndLine((TypeDeclaration) n);
+		}
 		return Util.getLineNumber(n.sourceEnd, lineseps, 0, lineseps.length - 1);
 		// return ProblemHandler.searchLineNumber(lineseps,
 		// currCompilationResult.lineSeparatorPositions,
