@@ -1,18 +1,13 @@
 /* *******************************************************************
- * Copyright (c) 2002 Palo Alto Research Center, Incorporated (PARC).
+ * Copyright (c) 2002-2010 Contributors
  * All rights reserved. 
  * This program and the accompanying materials are made available 
  * under the terms of the Eclipse Public License v1.0 
  * which accompanies this distribution and is available at 
  * http://www.eclipse.org/legal/epl-v10.html 
- *  
- * Contributors: 
- *     PARC     initial implementation 
  * ******************************************************************/
-
 package org.aspectj.weaver;
 
-import java.io.DataOutputStream;
 import java.io.IOException;
 import java.lang.reflect.Modifier;
 import java.util.ArrayList;
@@ -26,31 +21,35 @@ import java.util.Set;
 import org.aspectj.bridge.ISourceLocation;
 
 /**
- * This is the declared member, i.e. it will always correspond to an actual method/... declaration
+ * Represent a resolved member. Components of it are expected to exist. This member will correspond to a real member *unless* it is
+ * being used to represent the effect of an ITD.
+ * 
+ * @author PARC
+ * @author Andy Clement
  */
 public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, AnnotatedElement, TypeVariableDeclaringElement,
 		ResolvedMember {
 
 	private String[] parameterNames = null;
 	protected UnresolvedType[] checkedExceptions = UnresolvedType.NONE;
+
 	/**
 	 * if this member is a parameterized version of a member in a generic type, then this field holds a reference to the member we
 	 * parameterize.
 	 */
 	protected ResolvedMember backingGenericMember = null;
 
-	protected Set<ResolvedType> annotationTypes = null;
+	protected AnnotationAJ[] annotations = null;
+	protected ResolvedType[] annotationTypes = null;
+	protected AnnotationAJ[][] parameterAnnotations = null;
 	protected ResolvedType[][] parameterAnnotationTypes = null;
 
 	// Some members are 'created' to represent other things (for example ITDs).
-	// These
-	// members have their annotations stored elsewhere, and this flag indicates
-	// that is
-	// the case. It is up to the caller to work out where that is!
+	// These members have their annotations stored elsewhere, and this flag indicates
+	// that is the case. It is up to the caller to work out where that is!
 	// Once determined the caller may choose to stash the annotations in this
 	// member...
-	private boolean isAnnotatedElsewhere = false; // this field is not
-	// serialized.
+	private boolean isAnnotatedElsewhere = false;
 	private boolean isAjSynthetic = false;
 
 	// generic methods have type variables
@@ -106,7 +105,7 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 		ResolvedType originalDeclaringType = joinPointSignature.getDeclaringType().resolve(inAWorld);
 		ResolvedMemberImpl firstDefiningMember = (ResolvedMemberImpl) joinPointSignature.resolve(inAWorld);
 		if (firstDefiningMember == null) {
-			return new JoinPointSignature[0];
+			return JoinPointSignature.EMPTY_ARRAY;
 		}
 		// declaringType can be unresolved if we matched a synthetic member
 		// generated by Aj...
@@ -116,7 +115,7 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 		ResolvedType firstDefiningType = firstDefiningMember.getDeclaringType().resolve(inAWorld);
 		if (firstDefiningType != originalDeclaringType) {
 			if (joinPointSignature.getKind() == Member.CONSTRUCTOR) {
-				return new JoinPointSignature[0];
+				return JoinPointSignature.EMPTY_ARRAY;
 			}
 			// else if (shadowMember.isStatic()) {
 			// return new ResolvedMember[] {firstDefiningMember};
@@ -127,8 +126,7 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 		accumulateTypesInBetween(originalDeclaringType, firstDefiningType, declaringTypes);
 		Set<ResolvedMember> memberSignatures = new HashSet<ResolvedMember>();
 		for (ResolvedType declaringType : declaringTypes) {
-			ResolvedMember member = firstDefiningMember.withSubstituteDeclaringType(declaringType);
-			memberSignatures.add(member);
+			memberSignatures.add(new JoinPointSignature(firstDefiningMember, declaringType));
 		}
 
 		if (shouldWalkUpHierarchyFor(firstDefiningMember)) {
@@ -137,7 +135,7 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 			// every type between the firstDefiningMember and the root defining
 			// member.
 			Iterator<ResolvedType> superTypeIterator = firstDefiningType.getDirectSupertypes();
-			List typesAlreadyVisited = new ArrayList();
+			List<ResolvedType> typesAlreadyVisited = new ArrayList<ResolvedType>();
 			accumulateMembersMatching(firstDefiningMember, superTypeIterator, typesAlreadyVisited, memberSignatures, false);
 		}
 
@@ -147,12 +145,15 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 	}
 
 	private static boolean shouldWalkUpHierarchyFor(Member aMember) {
-		if (aMember.getKind() == Member.CONSTRUCTOR)
+		if (aMember.getKind() == Member.CONSTRUCTOR) {
 			return false;
-		if (aMember.getKind() == Member.FIELD)
+		}
+		if (aMember.getKind() == Member.FIELD) {
 			return false;
-		if (aMember.isStatic())
+		}
+		if (Modifier.isStatic(aMember.getModifiers())) {
 			return false;
+		}
 		return true;
 	}
 
@@ -193,8 +194,7 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 					accumulateTypesInBetween(toLookIn, resolvedDeclaringType, declaringTypes);
 					for (ResolvedType declaringType : declaringTypes) {
 						// typesAlreadyVisited.add(declaringType);
-						ResolvedMember member = foundMember.withSubstituteDeclaringType(declaringType);
-						foundMembers.add(member);
+						foundMembers.add(new JoinPointSignature(foundMember, declaringType));
 					}
 					if (!ignoreGenerics && toLookIn.isParameterizedType() && (foundMember.backingGenericMember != null)) {
 						foundMembers.add(new JoinPointSignature(foundMember.backingGenericMember, foundMember.declaringType
@@ -218,8 +218,9 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 	 * @return
 	 */
 	private static boolean isVisibleTo(ResolvedMember childMember, ResolvedMember parentMember) {
-		if (childMember.getDeclaringType().equals(parentMember.getDeclaringType()))
+		if (childMember.getDeclaringType().equals(parentMember.getDeclaringType())) {
 			return true;
+		}
 		if (Modifier.isPrivate(parentMember.getModifiers())) {
 			return false;
 		} else {
@@ -286,10 +287,14 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 			}
 			return backingGenericMember.hasAnnotation(ofType);
 		}
-		if (annotationTypes == null) {
-			return false;
+		if (annotationTypes != null) {
+			for (int i = 0, max = annotationTypes.length; i < max; i++) {
+				if (annotationTypes[i].equals(ofType)) {
+					return true;
+				}
+			}
 		}
-		return annotationTypes.contains(ofType);
+		return false;
 	}
 
 	public ResolvedType[] getAnnotationTypes() {
@@ -303,10 +308,7 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 			}
 			return backingGenericMember.getAnnotationTypes();
 		}
-		if (annotationTypes == null) {
-			return null;
-		}
-		return annotationTypes.toArray(new ResolvedType[] {});
+		return annotationTypes;
 	}
 
 	public String getAnnotationDefaultValue() {
@@ -316,41 +318,46 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 
 	@Override
 	public AnnotationAJ[] getAnnotations() {
-		if (backingGenericMember != null)
+		if (backingGenericMember != null) {
 			return backingGenericMember.getAnnotations();
+		}
 		return super.getAnnotations();
 	}
 
-	public void setAnnotationTypes(ResolvedType[] annotationtypes) {
-		if (annotationTypes == null) {
-			annotationTypes = new HashSet<ResolvedType>();
-		}
-		for (int i = 0; i < annotationtypes.length; i++) {
-			ResolvedType typeX = annotationtypes[i];
-			annotationTypes.add(typeX);
-		}
+	public void setAnnotationTypes(ResolvedType[] annotationTypes) {
+		this.annotationTypes = annotationTypes;
 	}
 
 	public ResolvedType[][] getParameterAnnotationTypes() {
-		if (parameterAnnotationTypes == null)
-			return null;
 		return parameterAnnotationTypes;
 	}
 
 	public AnnotationAJ[][] getParameterAnnotations() {
-		if (backingGenericMember != null)
+		if (backingGenericMember != null) {
 			return backingGenericMember.getParameterAnnotations();
+		}
 		throw new BCException("Cannot return parameter annotations for a " + this.getClass().getName() + " member");
 		// return super.getParameterAnnotations();
 	}
 
 	public void addAnnotation(AnnotationAJ annotation) {
-		// FIXME asc only allows for annotation types, not instances - should
-		// it?
 		if (annotationTypes == null) {
-			annotationTypes = new HashSet<ResolvedType>();
+			annotationTypes = new ResolvedType[1];
+			annotationTypes[0] = annotation.getType();
+			annotations = new AnnotationAJ[1];
+			annotations[0] = annotation;
+		} else {
+			int len = annotations.length;
+			AnnotationAJ[] ret = new AnnotationAJ[len + 1];
+			System.arraycopy(annotations, 0, ret, 0, len);
+			ret[len] = annotation;
+			annotations = ret;
+
+			ResolvedType[] newAnnotationTypes = new ResolvedType[len + 1];
+			System.arraycopy(annotationTypes, 0, newAnnotationTypes, 0, len);
+			newAnnotationTypes[len] = annotation.getType();
+			annotationTypes = newAnnotationTypes;
 		}
-		annotationTypes.add(annotation.getType());
 	}
 
 	public boolean isBridgeMethod() {
@@ -371,12 +378,30 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 		return (modifiers & 4096) != 0; // do we know better?
 	}
 
-	public void write(DataOutputStream s) throws IOException {
+	public void write(CompressingDataOutputStream s) throws IOException {
 		getKind().write(s);
-		getDeclaringType().write(s);
+		s.writeBoolean(s.canCompress()); // boolean indicates if parts of this are compressed references
+
+		// write out the signature of the declaring type of this member
+		if (s.canCompress()) {
+			s.writeCompressedSignature(getDeclaringType().getSignature());
+		} else {
+			getDeclaringType().write(s);
+		}
+
+		// write out the modifiers
 		s.writeInt(modifiers);
-		s.writeUTF(getName());
-		s.writeUTF(getSignature());
+
+		// write out the name and the signature of this member
+		if (s.canCompress()) {
+			s.writeCompressedName(getName());
+			s.writeCompressedSignature(getSignature());
+		} else {
+			s.writeUTF(getName());
+			s.writeUTF(getSignature());
+		}
+
+		// write out the array clauses
 		UnresolvedType.writeArray(getExceptions(), s);
 
 		s.writeInt(getStart());
@@ -385,24 +410,33 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 
 		// Write out any type variables...
 		if (typeVariables == null) {
-			s.writeInt(0);
+			s.writeByte(0);
 		} else {
-			s.writeInt(typeVariables.length);
+			s.writeByte(typeVariables.length);
 			for (int i = 0; i < typeVariables.length; i++) {
 				typeVariables[i].write(s);
 			}
 		}
 		String gsig = getGenericSignature();
+
+		// change this to a byte: 255=false 0>254 means true and encodes the number of parameters
 		if (getSignature().equals(gsig)) {
-			s.writeBoolean(false);
+			s.writeByte(0xff);
 		} else {
-			s.writeBoolean(true);
-			s.writeInt(parameterTypes.length);
+			s.writeByte(parameterTypes.length);
 			for (int i = 0; i < parameterTypes.length; i++) {
-				UnresolvedType array_element = parameterTypes[i];
-				array_element.write(s);
+				if (s.canCompress()) {
+					s.writeCompressedSignature(parameterTypes[i].getSignature());
+				} else {
+					UnresolvedType array_element = parameterTypes[i];
+					array_element.write(s);
+				}
+			}
+			if (s.canCompress()) {
+				s.writeCompressedSignature(returnType.getSignature());
+			} else {
+				returnType.write(s);
 			}
-			returnType.write(s);
 		}
 	}
 
@@ -452,7 +486,7 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 		return sb.toString();
 	}
 
-	public static void writeArray(ResolvedMember[] members, DataOutputStream s) throws IOException {
+	public static void writeArray(ResolvedMember[] members, CompressingDataOutputStream s) throws IOException {
 		s.writeInt(members.length);
 		for (int i = 0, len = members.length; i < len; i++) {
 			members[i].write(s);
@@ -462,8 +496,13 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 	public static ResolvedMemberImpl readResolvedMember(VersionedDataInputStream s, ISourceContext sourceContext)
 			throws IOException {
 
-		ResolvedMemberImpl m = new ResolvedMemberImpl(MemberKind.read(s), UnresolvedType.read(s), s.readInt(), s.readUTF(), s
-				.readUTF());
+		MemberKind mk = MemberKind.read(s);
+		boolean compressed = (s.isAtLeast169() ? s.readBoolean() : false);
+		UnresolvedType declaringType = compressed ? UnresolvedType.forSignature(s.readUtf8(s.readShort())) : UnresolvedType.read(s);
+		int modifiers = s.readInt();
+		String name = compressed ? s.readUtf8(s.readShort()) : s.readUTF();
+		String signature = compressed ? s.readUtf8(s.readShort()) : s.readUTF();
+		ResolvedMemberImpl m = new ResolvedMemberImpl(mk, declaringType, modifiers, name, signature);
 		m.checkedExceptions = UnresolvedType.readArray(s);
 
 		m.start = s.readInt();
@@ -474,11 +513,12 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 
 			if (s.getMajorVersion() >= AjAttribute.WeaverVersionInfo.WEAVER_VERSION_MAJOR_AJ150M4) {
 				boolean isvarargs = s.readBoolean();
-				if (isvarargs)
+				if (isvarargs) {
 					m.setVarargsMethod();
+				}
 			}
 
-			int tvcount = s.readInt();
+			int tvcount = s.isAtLeast169() ? s.readByte() : s.readInt();
 			if (tvcount != 0) {
 				m.typeVariables = new TypeVariable[tvcount];
 				for (int i = 0; i < tvcount; i++) {
@@ -487,14 +527,26 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 				}
 			}
 			if (s.getMajorVersion() >= AjAttribute.WeaverVersionInfo.WEAVER_VERSION_MAJOR_AJ150M4) {
-				boolean hasAGenericSignature = s.readBoolean();
+				int pcount = -1;
+				boolean hasAGenericSignature = false;
+				if (s.isAtLeast169()) {
+					pcount = s.readByte();
+					hasAGenericSignature = (pcount >= 0 && pcount < 255);
+				} else {
+					hasAGenericSignature = s.readBoolean();
+				}
 				if (hasAGenericSignature) {
-					int ps = s.readInt();
+					int ps = (s.isAtLeast169() ? pcount : s.readInt());
 					UnresolvedType[] params = new UnresolvedType[ps];
 					for (int i = 0; i < params.length; i++) {
-						params[i] = TypeFactory.createTypeFromSignature(s.readUTF());
+						if (compressed) {
+							params[i] = TypeFactory.createTypeFromSignature(s.readSignature());
+						} else {
+							params[i] = TypeFactory.createTypeFromSignature(s.readUTF());
+						}
 					}
-					UnresolvedType rt = TypeFactory.createTypeFromSignature(s.readUTF());
+					UnresolvedType rt = compressed ? TypeFactory.createTypeFromSignature(s.readSignature()) : TypeFactory
+							.createTypeFromSignature(s.readUTF());
 					m.parameterTypes = params;
 					m.returnType = rt;
 				}
@@ -534,8 +586,9 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 			// annotationTypes = r;
 			// }
 			declaringType = declaringType.resolve(world);
-			if (declaringType.isRawType())
+			if (declaringType.isRawType()) {
 				declaringType = ((ReferenceType) declaringType).getGenericType();
+			}
 
 			if (parameterTypes != null && parameterTypes.length > 0) {
 				for (int i = 0; i < parameterTypes.length; i++) {
@@ -614,16 +667,9 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 		return Modifier.isPublic(modifiers);
 	}
 
-	public boolean isProtected() {
-		return Modifier.isProtected(modifiers);
-	}
-
-	public boolean isNative() {
-		return Modifier.isNative(modifiers);
-	}
-
 	public boolean isDefault() {
-		return !(isPublic() || isProtected() || isPrivate());
+		int mods = getModifiers();
+		return !(Modifier.isPublic(mods) || Modifier.isProtected(mods) || Modifier.isPrivate(mods));
 	}
 
 	public boolean isVisible(ResolvedType fromType) {
@@ -670,9 +716,14 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 	 * turn into List<String> (for example) - if (!isParameterized) then List<T> will turn into List.
 	 */
 	public ResolvedMemberImpl parameterizedWith(UnresolvedType[] typeParameters, ResolvedType newDeclaringType,
-			boolean isParameterized, List aliases) {
+			boolean isParameterized, List<String> aliases) {
+		// PR308773
+		// this check had problems for the inner type of a generic type because the inner type can be represented
+		// by a 'simple type' if it is only sharing type variables with the outer and has none of its own. To avoid the
+		// check going bang in this case we check for $ (crap...) - we can't check the outer because the declaring type
+		// is considered unresolved...
 		if (// isParameterized && <-- might need this bit...
-		!getDeclaringType().isGenericType()) {
+		!getDeclaringType().isGenericType() && getDeclaringType().getName().indexOf("$") == -1) {
 			throw new IllegalStateException("Can't ask to parameterize a member of non-generic type: " + getDeclaringType()
 					+ "  kind(" + getDeclaringType().typeKind + ")");
 		}
@@ -698,8 +749,7 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 		// the same value as the type variables real name.
 		if (aliases != null) {
 			int posn = 0;
-			for (Iterator iter = aliases.iterator(); iter.hasNext();) {
-				String typeVariableAlias = (String) iter.next();
+			for (String typeVariableAlias : aliases) {
 				typeMap.put(typeVariableAlias, (!typeParametersSupplied ? typeVariables[posn].getFirstBound()
 						: typeParameters[posn]));
 				posn++;
@@ -727,7 +777,7 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 	 * Replace occurrences of type variables in the signature with values contained in the map. The map is of the form
 	 * A=String,B=Integer and so a signature List<A> Foo.m(B i) {} would become List<String> Foo.m(Integer i) {}
 	 */
-	public ResolvedMember parameterizedWith(Map m, World w) {
+	public ResolvedMember parameterizedWith(Map<String, UnresolvedType> m, World w) {
 		// if (//isParameterized && <-- might need this bit...
 		// !getDeclaringType().isGenericType()) {
 		// throw new IllegalStateException(
@@ -736,42 +786,43 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 		// getDeclaringType().typeKind+")");
 		// }
 		declaringType = declaringType.resolve(w);
-		if (declaringType.isRawType())
+		if (declaringType.isRawType()) {
 			declaringType = ((ResolvedType) declaringType).getGenericType();
-		// TypeVariable[] typeVariables = getDeclaringType().getTypeVariables();
-		// if (isParameterized && (typeVariables.length !=
-		// typeParameters.length)) {
-		// throw new
-		// IllegalStateException("Wrong number of type parameters supplied");
-		// }
-		// Map typeMap = new HashMap();
-		// boolean typeParametersSupplied = typeParameters!=null &&
-		// typeParameters.length>0;
-		// if (typeVariables!=null) {
-		// // If no 'replacements' were supplied in the typeParameters array
-		// then collapse
-		// // type variables to their first bound.
-		// for (int i = 0; i < typeVariables.length; i++) {
-		// UnresolvedType ut =
-		// (!typeParametersSupplied?typeVariables[i].getFirstBound
-		// ():typeParameters[i]);
-		// typeMap.put(typeVariables[i].getName(),ut);
-		// }
-		// }
-		// // For ITDs on generic types that use type variables from the target
-		// type, the aliases
-		// // record the alternative names used throughout the ITD expression
-		// that must map to
-		// // the same value as the type variables real name.
-		// if (aliases!=null) {
-		// int posn = 0;
-		// for (Iterator iter = aliases.iterator(); iter.hasNext();) {
-		// String typeVariableAlias = (String) iter.next();
-		// typeMap.put(typeVariableAlias,(!typeParametersSupplied?typeVariables[
-		// posn].getFirstBound():typeParameters[posn]));
-		// posn++;
-		// }
-		// }
+			// TypeVariable[] typeVariables = getDeclaringType().getTypeVariables();
+			// if (isParameterized && (typeVariables.length !=
+			// typeParameters.length)) {
+			// throw new
+			// IllegalStateException("Wrong number of type parameters supplied");
+			// }
+			// Map typeMap = new HashMap();
+			// boolean typeParametersSupplied = typeParameters!=null &&
+			// typeParameters.length>0;
+			// if (typeVariables!=null) {
+			// // If no 'replacements' were supplied in the typeParameters array
+			// then collapse
+			// // type variables to their first bound.
+			// for (int i = 0; i < typeVariables.length; i++) {
+			// UnresolvedType ut =
+			// (!typeParametersSupplied?typeVariables[i].getFirstBound
+			// ():typeParameters[i]);
+			// typeMap.put(typeVariables[i].getName(),ut);
+			// }
+			// }
+			// // For ITDs on generic types that use type variables from the target
+			// type, the aliases
+			// // record the alternative names used throughout the ITD expression
+			// that must map to
+			// // the same value as the type variables real name.
+			// if (aliases!=null) {
+			// int posn = 0;
+			// for (Iterator iter = aliases.iterator(); iter.hasNext();) {
+			// String typeVariableAlias = (String) iter.next();
+			// typeMap.put(typeVariableAlias,(!typeParametersSupplied?typeVariables[
+			// posn].getFirstBound():typeParameters[posn]));
+			// posn++;
+			// }
+			// }
+		}
 
 		UnresolvedType parameterizedReturnType = parameterize(getGenericReturnType(), m, true, w);
 		UnresolvedType[] parameterizedParameterTypes = new UnresolvedType[getGenericParameterTypes().length];
@@ -796,19 +847,20 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 		return typeVariables;
 	}
 
-	protected UnresolvedType parameterize(UnresolvedType aType, Map typeVariableMap, boolean inParameterizedType, World w) {
+	protected UnresolvedType parameterize(UnresolvedType aType, Map<String, UnresolvedType> typeVariableMap,
+			boolean inParameterizedType, World w) {
 		if (aType instanceof TypeVariableReference) {
 			String variableName = ((TypeVariableReference) aType).getTypeVariable().getName();
 			if (!typeVariableMap.containsKey(variableName)) {
 				return aType; // if the type variable comes from the method (and
 				// not the type) thats OK
 			}
-			return (UnresolvedType) typeVariableMap.get(variableName);
+			return typeVariableMap.get(variableName);
 		} else if (aType.isParameterizedType()) {
 			if (inParameterizedType) {
-				if (w != null)
+				if (w != null) {
 					aType = aType.resolve(w);
-				else {
+				} else {
 					UnresolvedType dType = getDeclaringType();
 					aType = aType.resolve(((ResolvedType) dType).getWorld());
 				}
@@ -909,31 +961,18 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 		returnType = UnresolvedType.OBJECTARRAY;
 	}
 
-	/**
-	 * Returns a copy of this member but with the declaring type swapped. Copy only needs to be shallow.
-	 * 
-	 * @param newDeclaringType
-	 */
-	public JoinPointSignature withSubstituteDeclaringType(ResolvedType newDeclaringType) {
-		JoinPointSignature ret = new JoinPointSignature(this, newDeclaringType);
-		return ret;
-	}
-
 	/**
 	 * Returns true if this member matches the other. The matching takes into account name and parameter types only. When comparing
 	 * parameter types, we allow any type variable to match any other type variable regardless of bounds.
 	 */
 	public boolean matches(ResolvedMember aCandidateMatch, boolean ignoreGenerics) {
-		// if (this.getName().equals("get")) {
-		// System.out.println("In RMI.matches() this=" + this + " candidate=" + aCandidateMatch);
-		// }
 		ResolvedMemberImpl candidateMatchImpl = (ResolvedMemberImpl) aCandidateMatch;
 		if (!getName().equals(aCandidateMatch.getName())) {
 			return false;
 		}
-		UnresolvedType[] myParameterTypes = getGenericParameterTypes();
+		UnresolvedType[] parameterTypes = getGenericParameterTypes();
 		UnresolvedType[] candidateParameterTypes = aCandidateMatch.getGenericParameterTypes();
-		if (myParameterTypes.length != candidateParameterTypes.length) {
+		if (parameterTypes.length != candidateParameterTypes.length) {
 			return false;
 		}
 		boolean b = false;
@@ -983,12 +1022,13 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 	// string suitable for
 	// comparison.
 	private String getParameterSigWithBoundsRemoved() {
-		if (myParameterSignatureWithBoundsRemoved != null)
+		if (myParameterSignatureWithBoundsRemoved != null) {
 			return myParameterSignatureWithBoundsRemoved;
+		}
 		StringBuffer sig = new StringBuffer();
 		UnresolvedType[] myParameterTypes = getGenericParameterTypes();
 		for (int i = 0; i < myParameterTypes.length; i++) {
-			appendSigWithTypeVarBoundsRemoved(myParameterTypes[i], sig, new HashSet());
+			appendSigWithTypeVarBoundsRemoved(myParameterTypes[i], sig, new HashSet<UnresolvedType>());
 		}
 		myParameterSignatureWithBoundsRemoved = sig.toString();
 		return myParameterSignatureWithBoundsRemoved;
@@ -999,8 +1039,9 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 	 * getParam
 	 */
 	public String getParameterSignatureErased() {
-		if (myParameterSignatureErasure != null)
+		if (myParameterSignatureErasure != null) {
 			return myParameterSignatureErasure;
+		}
 		StringBuffer sig = new StringBuffer();
 		UnresolvedType[] myParameterTypes = getParameterTypes();
 		for (int i = 0; i < myParameterTypes.length; i++) {
@@ -1024,7 +1065,8 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 	// does NOT produce a meaningful java signature, but does give a unique
 	// string suitable for
 	// comparison.
-	public static void appendSigWithTypeVarBoundsRemoved(UnresolvedType aType, StringBuffer toBuffer, Set alreadyUsedTypeVars) {
+	public static void appendSigWithTypeVarBoundsRemoved(UnresolvedType aType, StringBuffer toBuffer,
+			Set<UnresolvedType> alreadyUsedTypeVars) {
 		if (aType.isTypeVariableReference()) {
 			TypeVariableReferenceType typeVariableRT = (TypeVariableReferenceType) aType;
 			// pr204505
@@ -1032,7 +1074,7 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 				toBuffer.append("...");
 			} else {
 				alreadyUsedTypeVars.add(aType);
-				appendSigWithTypeVarBoundsRemoved(typeVariableRT.getUpperBound(), toBuffer, alreadyUsedTypeVars);
+				appendSigWithTypeVarBoundsRemoved(typeVariableRT.getTypeVariable().getFirstBound(), toBuffer, alreadyUsedTypeVars);
 			}
 			// toBuffer.append("T;");
 		} else if (aType.isParameterizedType()) {
@@ -1055,25 +1097,30 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 
 		// modifiers
 		int mods = modifiers;
-		if ((mods & 4096) > 0)
+		if ((mods & 4096) > 0) {
 			mods = mods - 4096; // remove synthetic (added in the ASM case but
+		}
 		// not in the BCEL case...)
-		if ((mods & 512) > 0)
+		if ((mods & 512) > 0) {
 			mods = mods - 512; // remove interface (added in the BCEL case but
+		}
 		// not in the ASM case...)
-		if ((mods & 131072) > 0)
+		if ((mods & 131072) > 0) {
 			mods = mods - 131072; // remove deprecated (added in the ASM case
+		}
 		// but not in the BCEL case...)
 		String modsStr = Modifier.toString(mods);
-		if (modsStr.length() != 0)
+		if (modsStr.length() != 0) {
 			r.append(modsStr).append("(" + mods + ")").append(" ");
+		}
 
 		// type variables
 		if (typeVariables != null && typeVariables.length > 0) {
 			r.append("<");
 			for (int i = 0; i < typeVariables.length; i++) {
-				if (i > 0)
+				if (i > 0) {
 					r.append(",");
+				}
 				TypeVariable t = typeVariables[i];
 				r.append(t.toDebugString());
 			}
@@ -1096,11 +1143,13 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 			boolean parameterNamesExist = showParameterNames && parameterNames != null && parameterNames.length == params.length;
 			if (params.length != 0) {
 				for (int i = 0, len = params.length; i < len; i++) {
-					if (i > 0)
+					if (i > 0) {
 						r.append(", ");
+					}
 					r.append(params[i].toDebugString());
-					if (parameterNamesExist)
+					if (parameterNamesExist) {
 						r.append(" ").append(parameterNames[i]);
+					}
 				}
 			}
 			r.append(")");
@@ -1135,22 +1184,27 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 	}
 
 	public boolean isCompatibleWith(Member am) {
-		if (kind != METHOD || am.getKind() != METHOD)
+		if (kind != METHOD || am.getKind() != METHOD) {
 			return true;
-		if (!name.equals(am.getName()))
+		}
+		if (!name.equals(am.getName())) {
 			return true;
-		if (!equalTypes(getParameterTypes(), am.getParameterTypes()))
+		}
+		if (!equalTypes(getParameterTypes(), am.getParameterTypes())) {
 			return true;
+		}
 		return getReturnType().equals(am.getReturnType());
 	}
 
 	private static boolean equalTypes(UnresolvedType[] a, UnresolvedType[] b) {
 		int len = a.length;
-		if (len != b.length)
+		if (len != b.length) {
 			return false;
+		}
 		for (int i = 0; i < len; i++) {
-			if (!a[i].equals(b[i]))
+			if (!a[i].equals(b[i])) {
 				return false;
+			}
 		}
 		return true;
 	}
@@ -1159,8 +1213,9 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 		// Check locally...
 		if (typeVariables != null) {
 			for (int i = 0; i < typeVariables.length; i++) {
-				if (typeVariables[i].getName().equals(name))
+				if (typeVariables[i].getName().equals(name)) {
 					return typeVariables[i];
+				}
 			}
 		}
 		// check the declaring type!
