@@ -28,6 +28,7 @@ import java.util.Properties;
 import java.util.Set;
 import java.util.WeakHashMap;
 
+import org.aspectj.bridge.IMessage;
 import org.aspectj.bridge.IMessageHandler;
 import org.aspectj.bridge.ISourceLocation;
 import org.aspectj.bridge.Message;
@@ -37,7 +38,10 @@ import org.aspectj.bridge.context.PinpointingMessageHandler;
 import org.aspectj.util.IStructureModel;
 import org.aspectj.weaver.UnresolvedType.TypeKind;
 import org.aspectj.weaver.patterns.DeclareAnnotation;
+import org.aspectj.weaver.patterns.DeclareParents;
 import org.aspectj.weaver.patterns.DeclarePrecedence;
+import org.aspectj.weaver.patterns.DeclareSoft;
+import org.aspectj.weaver.patterns.DeclareTypeErrorOrWarning;
 import org.aspectj.weaver.patterns.Pointcut;
 import org.aspectj.weaver.patterns.TypePattern;
 import org.aspectj.weaver.tools.PointcutDesignatorHandler;
@@ -65,7 +69,7 @@ public abstract class World implements Dump.INode {
 	protected TypeMap typeMap = new TypeMap(this); // Signature to ResolvedType
 
 	/** New pointcut designators this world supports */
-	private Set pointcutDesignators;
+	private Set<PointcutDesignatorHandler> pointcutDesignators;
 
 	// see pr145963
 	/** Should we create the hierarchy for binary classes and aspects */
@@ -100,6 +104,10 @@ public abstract class World implements Dump.INode {
 	/** When behaving in a Java 5 way autoboxing is considered */
 	private boolean behaveInJava5Way = false;
 
+	/** Should timing information be reported (as info messages)? */
+	private boolean timing = false;
+	private boolean timingPeriodically = true;
+
 	/** Determines if this world could be used for multiple compiles */
 	private boolean incrementalCompileCouldFollow = false;
 
@@ -121,13 +129,21 @@ public abstract class World implements Dump.INode {
 	private boolean synchronizationPointcutsInUse = false;
 	// Xset'table options
 	private boolean runMinimalMemory = false;
+	private boolean runMinimalMemorySet = false;
 	private boolean shouldPipelineCompilation = true;
 	private boolean shouldGenerateStackMaps = false;
 	protected boolean bcelRepositoryCaching = xsetBCEL_REPOSITORY_CACHING_DEFAULT.equalsIgnoreCase("true");
 	private boolean fastMethodPacking = false;
+	private int itdVersion = 2; // defaults to 2nd generation itds
 	private boolean completeBinaryTypes = false;
+	private boolean overWeaving = false;
 	public boolean forDEBUG_structuralChangesCode = false;
 	public boolean forDEBUG_bridgingCode = false;
+	public boolean optimizedMatching = true;
+	protected long timersPerJoinpoint = 25000;
+	protected long timersPerType = 250;
+
+	public int infoMessagesEnabled = 0; // 0=uninitialized, 1=no, 2=yes
 
 	private static Trace trace = TraceFactory.getTraceFactory().getTrace(World.class);
 
@@ -267,7 +283,7 @@ public abstract class World implements Dump.INode {
 			// fault in generic wildcard, can't be done earlier because of init
 			// issues
 			// TODO ought to be shared single instance representing this
-			ResolvedType something = new BoundedReferenceType("*", "Ljava/lang/Object", this);
+			ResolvedType something = getWildcard();
 			typeMap.put("?", something);
 			return something;
 		}
@@ -301,6 +317,16 @@ public abstract class World implements Dump.INode {
 		}
 	}
 
+	// Only need one representation of '?' in a world - can be shared
+	private BoundedReferenceType wildcard;
+
+	private BoundedReferenceType getWildcard() {
+		if (wildcard == null) {
+			wildcard = new BoundedReferenceType(this);
+		}
+		return wildcard;
+	}
+
 	/**
 	 * Called when a type is resolved - enables its type hierarchy to be finished off before we proceed
 	 */
@@ -343,13 +369,42 @@ public abstract class World implements Dump.INode {
 		}
 		ResolvedType resolved = typeMap.get(ty.getSignature());
 		if (resolved == null) {
-			typeMap.put(ty.getSignature(), ty);
+			resolved = ensureRawTypeIfNecessary(ty);
+			typeMap.put(ty.getSignature(), resolved);
 			resolved = ty;
 		}
 		resolved.world = this;
 		return resolved;
 	}
 
+	/**
+	 * When the world is operating in 1.5 mode, the TypeMap should only contain RAW types and never directly generic types. The RAW
+	 * type will contain a reference to the generic type.
+	 * 
+	 * @param type a possibly generic type for which the raw needs creating as it is not currently in the world
+	 * @return a type suitable for putting into the world
+	 */
+	private ResolvedType ensureRawTypeIfNecessary(ResolvedType type) {
+		if (!isInJava5Mode()) {
+			// Don't care, not running in 1.5 mode
+			return type;
+		}
+		if (type.isRawType()) {
+			return type;
+		}
+		// Key requirement here is if it is generic, create a RAW entry to be put in the map that points to it
+		if (type instanceof ReferenceType && ((ReferenceType) type).getDelegate() != null && type.isGenericType()) {
+			ReferenceType rawType = new ReferenceType(type.getSignature(), this);
+			rawType.typeKind = UnresolvedType.TypeKind.RAW;
+			ReferenceTypeDelegate delegate = ((ReferenceType) type).getDelegate();
+			rawType.setDelegate(delegate);
+			rawType.setGenericType((ReferenceType) type);
+			return rawType;
+		}
+		// probably parameterized...
+		return type;
+	}
+
 	/**
 	 * Convenience method for finding a type by name and resolving it in one step.
 	 */
@@ -497,7 +552,7 @@ public abstract class World implements Dump.INode {
 			ret = new BoundedReferenceType(lowerBound, false, this);
 		} else {
 			// must be ? on its own!
-			ret = new BoundedReferenceType("*", "Ljava/lang/Object", this);
+			ret = getWildcard();
 		}
 		return ret;
 	}
@@ -569,6 +624,16 @@ public abstract class World implements Dump.INode {
 		return declaring.lookupSyntheticMember(member);
 	}
 
+	private boolean allLintIgnored = false;
+
+	public void setAllLintIgnored() {
+		allLintIgnored = true;
+	}
+
+	public boolean areAllLintIgnored() {
+		return allLintIgnored;
+	}
+
 	public abstract IWeavingSupport getWeavingSupport();
 
 	/**
@@ -580,10 +645,10 @@ public abstract class World implements Dump.INode {
 	 * Create an advice shadow munger for the given advice kind
 	 */
 	public final Advice createAdviceMunger(AdviceKind kind, Pointcut p, Member signature, int extraParameterFlags,
-			IHasSourceLocation loc) {
+			IHasSourceLocation loc, ResolvedType declaringAspect) {
 		AjAttribute.AdviceAttribute attribute = new AjAttribute.AdviceAttribute(kind, p, extraParameterFlags, loc.getStart(), loc
 				.getEnd(), loc.getSourceContext());
-		return getWeavingSupport().createAdviceMunger(attribute, p, signature);
+		return getWeavingSupport().createAdviceMunger(attribute, p, signature, declaringAspect);
 	}
 
 	/**
@@ -656,15 +721,15 @@ public abstract class World implements Dump.INode {
 		return typeVariableLookupScope;
 	}
 
-	public List getDeclareParents() {
+	public List<DeclareParents> getDeclareParents() {
 		return crosscuttingMembersSet.getDeclareParents();
 	}
 
-	public List getDeclareAnnotationOnTypes() {
+	public List<DeclareAnnotation> getDeclareAnnotationOnTypes() {
 		return crosscuttingMembersSet.getDeclareAnnotationOnTypes();
 	}
 
-	public List getDeclareAnnotationOnFields() {
+	public List<DeclareAnnotation> getDeclareAnnotationOnFields() {
 		return crosscuttingMembersSet.getDeclareAnnotationOnFields();
 	}
 
@@ -672,7 +737,11 @@ public abstract class World implements Dump.INode {
 		return crosscuttingMembersSet.getDeclareAnnotationOnMethods();
 	}
 
-	public List getDeclareSoft() {
+	public List<DeclareTypeErrorOrWarning> getDeclareTypeEows() {
+		return crosscuttingMembersSet.getDeclareTypeEows();
+	}
+
+	public List<DeclareSoft> getDeclareSoft() {
 		return crosscuttingMembersSet.getDeclareSofts();
 	}
 
@@ -732,6 +801,15 @@ public abstract class World implements Dump.INode {
 		behaveInJava5Way = b;
 	}
 
+	/**
+	 * Set the timing option (whether to collect timing info), this will also need INFO messages turned on for the message handler
+	 * being used. The reportPeriodically flag should be set to false under AJDT so numbers just come out at the end.
+	 */
+	public void setTiming(boolean timersOn, boolean reportPeriodically) {
+		timing = timersOn;
+		timingPeriodically = reportPeriodically;
+	}
+
 	/**
 	 * Set the error and warning threashold which can be taken from CompilerOptions (see bug 129282)
 	 * 
@@ -783,6 +861,13 @@ public abstract class World implements Dump.INode {
 		ensureAdvancedConfigurationProcessed();
 	}
 
+	public boolean areInfoMessagesEnabled() {
+		if (infoMessagesEnabled == 0) {
+			infoMessagesEnabled = (messageHandler.isIgnoring(IMessage.INFO) ? 1 : 2);
+		}
+		return infoMessagesEnabled == 2;
+	}
+
 	/**
 	 * may return null
 	 */
@@ -818,14 +903,24 @@ public abstract class World implements Dump.INode {
 	public final static String xsetTYPE_DEMOTION_DEBUG = "typeDemotionDebug";
 	public final static String xsetTYPE_REFS = "useWeakTypeRefs";
 	public final static String xsetBCEL_REPOSITORY_CACHING_DEFAULT = "true";
-	public final static String xsetFAST_PACK_METHODS = "fastPackMethods"; // default
-
-	// TRUE
+	public final static String xsetFAST_PACK_METHODS = "fastPackMethods"; // default true
+	public final static String xsetOVERWEAVING = "overWeaving";
+	public final static String xsetOPTIMIZED_MATCHING = "optimizedMatching";
+	public final static String xsetTIMERS_PER_JOINPOINT = "timersPerJoinpoint";
+	public final static String xsetTIMERS_PER_FASTMATCH_CALL = "timersPerFastMatchCall";
+	public final static String xsetITD_VERSION = "itdVersion";
+	public final static String xsetITD_VERSION_ORIGINAL = "1";
+	public final static String xsetITD_VERSION_2NDGEN = "2";
+	public final static String xsetITD_VERSION_DEFAULT = xsetITD_VERSION_2NDGEN;
 
 	public boolean isInJava5Mode() {
 		return behaveInJava5Way;
 	}
 
+	public boolean isTimingEnabled() {
+		return timing;
+	}
+
 	public void setTargetAspectjRuntimeLevel(String s) {
 		targetAspectjRuntimeLevel = s;
 	}
@@ -871,11 +966,11 @@ public abstract class World implements Dump.INode {
 	 * Map of types in the world, can have 'references' to expendable ones which can be garbage collected to recover memory. An
 	 * expendable type is a reference type that is not exposed to the weaver (ie just pulled in for type resolution purposes).
 	 */
-	protected static class TypeMap {
+	public static class TypeMap {
 
 		private static boolean debug = false;
 
-		private boolean demotionSystemActive = false;
+		private boolean demotionSystemActive;
 		private boolean debugDemotion = false;
 
 		// Strategy for entries in the expendable map
@@ -885,7 +980,7 @@ public abstract class World implements Dump.INode {
 		// memory
 		List<String> addedSinceLastDemote;
 
-		public int policy = USE_SOFT_REFS;
+		public int policy = USE_WEAK_REFS;
 
 		// Map of types that never get thrown away
 		final Map<String, ResolvedType> tMap = new HashMap<String, ResolvedType>();
@@ -904,12 +999,23 @@ public abstract class World implements Dump.INode {
 		private static Trace trace = TraceFactory.getTraceFactory().getTrace(World.TypeMap.class);
 
 		TypeMap(World w) {
+			demotionSystemActive = w.isDemotionActive();
 			addedSinceLastDemote = new ArrayList<String>();
 			this.w = w;
 			memoryProfiling = false;// !w.getMessageHandler().isIgnoring(Message.
 			// INFO);
 		}
 
+		// For testing
+		public Map getExpendableMap() {
+			return expendableMap;
+		}
+
+		// For testing
+		public Map getMainMap() {
+			return tMap;
+		}
+
 		/**
 		 * Go through any types added during the previous file weave. If any are suitable for demotion, then put them in the
 		 * expendable map where GC can claim them at some point later. Demotion means: the type is not an aspect, the type is not
@@ -1223,6 +1329,10 @@ public abstract class World implements Dump.INode {
 	// --- with java5 we can get into a recursive mess if we aren't careful when
 	// resolving types (*cough* java.lang.Enum) ---
 
+	public boolean isDemotionActive() {
+		return false;
+	}
+
 	// --- this first map is for java15 delegates which may try and recursively
 	// access the same type variables.
 	// --- I would rather stash this against a reference type - but we don't
@@ -1256,6 +1366,7 @@ public abstract class World implements Dump.INode {
 	}
 
 	public void ensureAdvancedConfigurationProcessed() {
+
 		// Check *once* whether the user has switched asm support off
 		if (!checkedAdvancedConfiguration) {
 			Properties p = getExtraConfiguration();
@@ -1269,6 +1380,14 @@ public abstract class World implements Dump.INode {
 									.info("[bcelRepositoryCaching=false] AspectJ will not use a bcel cache for class information"));
 				}
 
+				// ITD Versions
+				// 1 is the first version in use up to AspectJ 1.6.8
+				// 2 is from 1.6.9 onwards
+				s = p.getProperty(xsetITD_VERSION, xsetITD_VERSION_DEFAULT);
+				if (s.equals(xsetITD_VERSION_ORIGINAL)) {
+					itdVersion = 1;
+				}
+
 				s = p.getProperty(xsetFAST_PACK_METHODS, "true");
 				fastMethodPacking = s.equalsIgnoreCase("true");
 
@@ -1285,20 +1404,26 @@ public abstract class World implements Dump.INode {
 							MessageUtil.info("[completeBinaryTypes=true] Completion of binary types activated"));
 				}
 
-				s = p.getProperty(xsetTYPE_DEMOTION, "false");
+				s = p.getProperty(xsetTYPE_DEMOTION, "true"); // default is ON
+				if (s.equalsIgnoreCase("false")) {
+					typeMap.demotionSystemActive = false;
+				}
+
+				s = p.getProperty(xsetOVERWEAVING, "false");
 				if (s.equalsIgnoreCase("true")) {
-					typeMap.demotionSystemActive = true;
+					overWeaving = true;
 				}
 
 				s = p.getProperty(xsetTYPE_DEMOTION_DEBUG, "false");
 				if (s.equalsIgnoreCase("true")) {
 					typeMap.debugDemotion = true;
 				}
-				s = p.getProperty(xsetTYPE_REFS, "false");
-				if (s.equalsIgnoreCase("true")) {
-					typeMap.policy = TypeMap.USE_WEAK_REFS;
+				s = p.getProperty(xsetTYPE_REFS, "true");
+				if (s.equalsIgnoreCase("false")) {
+					typeMap.policy = TypeMap.USE_SOFT_REFS;
 				}
 
+				runMinimalMemorySet = p.getProperty(xsetRUN_MINIMAL_MEMORY) != null;
 				s = p.getProperty(xsetRUN_MINIMAL_MEMORY, "false");
 				runMinimalMemory = s.equalsIgnoreCase("true");
 				// if (runMinimalMemory)
@@ -1312,6 +1437,38 @@ public abstract class World implements Dump.INode {
 				s = p.getProperty(xsetDEBUG_BRIDGING, "false");
 				forDEBUG_bridgingCode = s.equalsIgnoreCase("true");
 
+				s = p.getProperty(xsetOPTIMIZED_MATCHING, "true");
+				optimizedMatching = s.equalsIgnoreCase("true");
+				if (!optimizedMatching) {
+					getMessageHandler().handleMessage(MessageUtil.info("[optimizedMatching=false] optimized matching turned off"));
+				}
+
+				s = p.getProperty(xsetTIMERS_PER_JOINPOINT, "25000");
+				try {
+					timersPerJoinpoint = Integer.parseInt(s);
+				} catch (Exception e) {
+					getMessageHandler().handleMessage(MessageUtil.error("unable to process timersPerJoinpoint value of " + s));
+					timersPerJoinpoint = 25000;
+				}
+
+				s = p.getProperty(xsetTIMERS_PER_FASTMATCH_CALL, "250");
+				try {
+					timersPerType = Integer.parseInt(s);
+				} catch (Exception e) {
+					getMessageHandler().handleMessage(MessageUtil.error("unable to process timersPerType value of " + s));
+					timersPerType = 250;
+				}
+
+			}
+			try {
+				String value = System.getProperty("aspectj.overweaving", "false");
+				if (value.equalsIgnoreCase("true")) {
+					System.out.println("ASPECTJ: aspectj.overweaving=true: overweaving switched ON");
+					overWeaving = true;
+				}
+			} catch (Throwable t) {
+				System.err.println("ASPECTJ: Unable to read system properties");
+				t.printStackTrace();
 			}
 			checkedAdvancedConfiguration = true;
 		}
@@ -1322,6 +1479,11 @@ public abstract class World implements Dump.INode {
 		return runMinimalMemory;
 	}
 
+	public boolean isRunMinimalMemorySet() {
+		ensureAdvancedConfigurationProcessed();
+		return runMinimalMemorySet;
+	}
+
 	public boolean shouldFastPackMethods() {
 		ensureAdvancedConfigurationProcessed();
 		return fastMethodPacking;
@@ -1366,14 +1528,14 @@ public abstract class World implements Dump.INode {
 	 */
 	public void registerPointcutHandler(PointcutDesignatorHandler designatorHandler) {
 		if (pointcutDesignators == null) {
-			pointcutDesignators = new HashSet();
+			pointcutDesignators = new HashSet<PointcutDesignatorHandler>();
 		}
 		pointcutDesignators.add(designatorHandler);
 	}
 
-	public Set getRegisteredPointcutHandlers() {
+	public Set<PointcutDesignatorHandler> getRegisteredPointcutHandlers() {
 		if (pointcutDesignators == null) {
-			return Collections.EMPTY_SET;
+			return Collections.emptySet();
 		}
 		return pointcutDesignators;
 	}
@@ -1382,6 +1544,10 @@ public abstract class World implements Dump.INode {
 
 	}
 
+	public boolean isOverWeaving() {
+		return overWeaving;
+	}
+
 	public void reportCheckerMatch(Checker checker, Shadow shadow) {
 	}
 
@@ -1396,6 +1562,16 @@ public abstract class World implements Dump.INode {
 		return true;
 	}
 
+    /**
+     * Determine if the named aspect requires a particular type around in order to
+     * be useful.  The type is named in the aop.xml file against the aspect.
+     *
+     * @return true if there is a type missing that this aspect really needed around
+     */
+	public boolean hasUnsatisfiedDependency(ResolvedType aspectType) {
+		return false;
+	}
+
 	public TypePattern getAspectScope(ResolvedType declaringType) {
 		return null;
 	}
@@ -1442,4 +1618,186 @@ public abstract class World implements Dump.INode {
 		return exclusionMap;
 	}
 
+	private TimeCollector timeCollector = null;
+
+	/**
+	 * Record the time spent matching a pointcut - this will accumulate over the lifetime of this world/weaver and be reported every
+	 * 25000 join points.
+	 */
+	public void record(Pointcut pointcut, long timetaken) {
+		if (timeCollector == null) {
+			ensureAdvancedConfigurationProcessed();
+			timeCollector = new TimeCollector(this);
+		}
+		timeCollector.record(pointcut, timetaken);
+	}
+
+	/**
+	 * Record the time spent fastmatching a pointcut - this will accumulate over the lifetime of this world/weaver and be reported
+	 * every 250 types.
+	 */
+	public void recordFastMatch(Pointcut pointcut, long timetaken) {
+		if (timeCollector == null) {
+			ensureAdvancedConfigurationProcessed();
+			timeCollector = new TimeCollector(this);
+		}
+		timeCollector.recordFastMatch(pointcut, timetaken);
+	}
+
+	public void reportTimers() {
+		if (timeCollector != null && !timingPeriodically) {
+			timeCollector.report();
+			timeCollector = new TimeCollector(this);
+		}
+	}
+
+	private static class TimeCollector {
+		private World world;
+		long joinpointCount;
+		long typeCount;
+		long perJoinpointCount;
+		long perTypes;
+		Map<String, Long> joinpointsPerPointcut = new HashMap<String, Long>();
+		Map<String, Long> timePerPointcut = new HashMap<String, Long>();
+		Map<String, Long> fastMatchTimesPerPointcut = new HashMap<String, Long>();
+		Map<String, Long> fastMatchTypesPerPointcut = new HashMap<String, Long>();
+
+		TimeCollector(World world) {
+			this.perJoinpointCount = world.timersPerJoinpoint;
+			this.perTypes = world.timersPerType;
+			this.world = world;
+			this.joinpointCount = 0;
+			this.typeCount = 0;
+			this.joinpointsPerPointcut = new HashMap<String, Long>();
+			this.timePerPointcut = new HashMap<String, Long>();
+		}
+
+		public void report() {
+			long totalTime = 0L;
+			for (String p : joinpointsPerPointcut.keySet()) {
+				totalTime += timePerPointcut.get(p);
+			}
+			world.getMessageHandler().handleMessage(
+					MessageUtil.info("Pointcut matching cost (total=" + (totalTime / 1000000) + "ms for " + joinpointCount
+							+ " joinpoint match calls):"));
+			for (String p : joinpointsPerPointcut.keySet()) {
+				StringBuffer sb = new StringBuffer();
+				sb.append("Time:" + (timePerPointcut.get(p) / 1000000) + "ms (jps:#" + joinpointsPerPointcut.get(p)
+						+ ") matching against " + p);
+				world.getMessageHandler().handleMessage(MessageUtil.info(sb.toString()));
+			}
+			world.getMessageHandler().handleMessage(MessageUtil.info("---"));
+
+			totalTime = 0L;
+			for (String p : fastMatchTimesPerPointcut.keySet()) {
+				totalTime += fastMatchTimesPerPointcut.get(p);
+			}
+			world.getMessageHandler().handleMessage(
+					MessageUtil.info("Pointcut fast matching cost (total=" + (totalTime / 1000000) + "ms for " + typeCount
+							+ " fast match calls):"));
+			for (String p : fastMatchTimesPerPointcut.keySet()) {
+				StringBuffer sb = new StringBuffer();
+				sb.append("Time:" + (fastMatchTimesPerPointcut.get(p) / 1000000) + "ms (types:#" + fastMatchTypesPerPointcut.get(p)
+						+ ") fast matching against " + p);
+				world.getMessageHandler().handleMessage(MessageUtil.info(sb.toString()));
+			}
+			world.getMessageHandler().handleMessage(MessageUtil.info("---"));
+
+		}
+
+		void record(Pointcut pointcut, long timetakenInNs) {
+			joinpointCount++;
+			String pointcutText = pointcut.toString();
+			Long jpcounter = joinpointsPerPointcut.get(pointcutText);
+			if (jpcounter == null) {
+				jpcounter = 1L;
+			} else {
+				jpcounter++;
+			}
+			joinpointsPerPointcut.put(pointcutText, jpcounter);
+
+			Long time = timePerPointcut.get(pointcutText);
+			if (time == null) {
+				time = timetakenInNs;
+			} else {
+				time += timetakenInNs;
+			}
+			timePerPointcut.put(pointcutText, time);
+			if (world.timingPeriodically) {
+				if ((joinpointCount % perJoinpointCount) == 0) {
+					long totalTime = 0L;
+					for (String p : joinpointsPerPointcut.keySet()) {
+						totalTime += timePerPointcut.get(p);
+					}
+					world.getMessageHandler().handleMessage(
+							MessageUtil.info("Pointcut matching cost (total=" + (totalTime / 1000000) + "ms for " + joinpointCount
+									+ " joinpoint match calls):"));
+					for (String p : joinpointsPerPointcut.keySet()) {
+						StringBuffer sb = new StringBuffer();
+						sb.append("Time:" + (timePerPointcut.get(p) / 1000000) + "ms (jps:#" + joinpointsPerPointcut.get(p)
+								+ ") matching against " + p);
+						world.getMessageHandler().handleMessage(MessageUtil.info(sb.toString()));
+					}
+					world.getMessageHandler().handleMessage(MessageUtil.info("---"));
+				}
+			}
+		}
+
+		void recordFastMatch(Pointcut pointcut, long timetakenInNs) {
+			typeCount++;
+			String pointcutText = pointcut.toString();
+			Long typecounter = fastMatchTypesPerPointcut.get(pointcutText);
+			if (typecounter == null) {
+				typecounter = 1L;
+			} else {
+				typecounter++;
+			}
+			fastMatchTypesPerPointcut.put(pointcutText, typecounter);
+
+			Long time = fastMatchTimesPerPointcut.get(pointcutText);
+			if (time == null) {
+				time = timetakenInNs;
+			} else {
+				time += timetakenInNs;
+			}
+			fastMatchTimesPerPointcut.put(pointcutText, time);
+			if (world.timingPeriodically) {
+				if ((typeCount % perTypes) == 0) {
+					long totalTime = 0L;
+					for (String p : fastMatchTimesPerPointcut.keySet()) {
+						totalTime += fastMatchTimesPerPointcut.get(p);
+					}
+					world.getMessageHandler().handleMessage(
+							MessageUtil.info("Pointcut fast matching cost (total=" + (totalTime / 1000000) + "ms for " + typeCount
+									+ " fast match calls):"));
+					for (String p : fastMatchTimesPerPointcut.keySet()) {
+						StringBuffer sb = new StringBuffer();
+						sb.append("Time:" + (fastMatchTimesPerPointcut.get(p) / 1000000) + "ms (types:#"
+								+ fastMatchTypesPerPointcut.get(p) + ") fast matching against " + p);
+						world.getMessageHandler().handleMessage(MessageUtil.info(sb.toString()));
+					}
+					world.getMessageHandler().handleMessage(MessageUtil.info("---"));
+				}
+			}
+		}
+	}
+
+	public TypeMap getTypeMap() {
+		return typeMap;
+	}
+
+	public static void reset() {
+		ResolvedType.resetPrimitives();
+	}
+
+	/**
+	 * Returns the version of ITD that this world wants to create. The default is the new style (2) but in some cases where there
+	 * might be a clash, the old style can be used. It is set through the option -Xset:itdVersion=1
+	 * 
+	 * @return the ITD version this world wants to create - 1=oldstyle 2=new, transparent style
+	 */
+	public int getItdVersion() {
+		return itdVersion;
+	}
+
 }
\ No newline at end of file
