@@ -185,8 +185,8 @@ public class AtAjAttributes {
 	 * @param msgHandler
 	 * @return list of AjAttributes
 	 */
-	public static List readAj5ClassAttributes(AsmManager model, JavaClass javaClass, ReferenceType type, ISourceContext context,
-			IMessageHandler msgHandler, boolean isCodeStyleAspect) {
+	public static List<AjAttribute> readAj5ClassAttributes(AsmManager model, JavaClass javaClass, ReferenceType type,
+			ISourceContext context, IMessageHandler msgHandler, boolean isCodeStyleAspect) {
 		boolean ignoreThisClass = javaClass.getClassName().charAt(0) == 'o'
 				&& javaClass.getClassName().startsWith("org.aspectj.lang.annotation");
 		if (ignoreThisClass) {
@@ -468,11 +468,11 @@ public class AtAjAttributes {
 	 * @param msgHandler
 	 * @return list of AjAttributes, always empty for now
 	 */
-	public static List readAj5FieldAttributes(Field field, BcelField bField, ResolvedType type, ISourceContext context,
-			IMessageHandler msgHandler) {
+	public static List<AjAttribute> readAj5FieldAttributes(Field field, BcelField bField, ResolvedType type,
+			ISourceContext context, IMessageHandler msgHandler) {
 		// Note: field annotation are for ITD and DEOW - processed at class
 		// level directly
-		return Collections.EMPTY_LIST;
+		return Collections.emptyList();
 	}
 
 	/**
@@ -779,9 +779,14 @@ public class AtAjAttributes {
 
 					// then iterate on field interface hierarchy (not object)
 					boolean hasAtLeastOneMethod = false;
-					ResolvedMember[] methods = fieldType.getMethodsWithoutIterator(true, false).toArray(new ResolvedMember[0]);
-					for (int i = 0; i < methods.length; i++) {
-						ResolvedMember method = methods[i];
+					Iterator<ResolvedMember> methodIterator = fieldType.getMethodsIncludingIntertypeDeclarations(false, true);
+					while (methodIterator.hasNext()) {
+						ResolvedMember method = methodIterator.next();
+
+						// ResolvedMember[] methods = fieldType.getMethodsWithoutIterator(true, false, false).toArray(
+						// new ResolvedMember[0]);
+						// for (int i = 0; i < methods.length; i++) {
+						// ResolvedMember method = methods[i];
 						if (method.isAbstract()) {
 							// moved to be detected at weave time if the target
 							// doesnt implement the methods
@@ -970,11 +975,15 @@ public class AtAjAttributes {
 			ResolvedType typeForDelegation = (ResolvedType) iterator.next();
 			// TODO check for overlapping interfaces. Eg. A implements I, I extends J - if they specify interfaces={I,J} we dont
 			// want to do any methods twice
-			ResolvedMember[] methods = typeForDelegation.getMethodsWithoutIterator(true, false).toArray(new ResolvedMember[0]);
+			ResolvedMember[] methods = typeForDelegation.getMethodsWithoutIterator(true, false, false).toArray(
+					new ResolvedMember[0]);
 			for (int i = 0; i < methods.length; i++) {
 				ResolvedMember method = methods[i];
 				if (method.isAbstract()) {
 					hasAtLeastOneMethod = true;
+					if (method.hasBackingGenericMember()) {
+						method = method.getBackingGenericMember();
+					}
 					MethodDelegateTypeMunger mdtm = new MethodDelegateTypeMunger(method, struct.enclosingType, "",
 							targetTypePattern, struct.method.getName(), struct.method.getSignature());
 					mdtm.setFieldType(methodReturnType);
@@ -1707,14 +1716,33 @@ public class AtAjAttributes {
 		final List<MethodArgument> arguments = new ArrayList<MethodArgument>();
 		LocalVariableTable lt = method.getLocalVariableTable();
 		if (lt != null) {
-			for (int j = 0; j < lt.getLocalVariableTable().length; j++) {
-				LocalVariable localVariable = lt.getLocalVariableTable()[j];
+			LocalVariable[] lvt = lt.getLocalVariableTable();
+			for (int j = 0; j < lvt.length; j++) {
+				LocalVariable localVariable = lvt[j];
 				if (localVariable.getStartPC() == 0) {
 					if (localVariable.getIndex() >= startAtStackIndex) {
 						arguments.add(new MethodArgument(localVariable.getName(), localVariable.getIndex()));
 					}
 				}
 			}
+			if (arguments.size() == 0) {
+				// could be cobertura code where some extra bytecode has been stuffed in at the start of the method
+				// but the local variable table hasn't been repaired - for example:
+				// LocalVariable(start_pc = 6, length = 40, index = 0:com.example.ExampleAspect this)
+				// LocalVariable(start_pc = 6, length = 40, index = 1:org.aspectj.lang.ProceedingJoinPoint pjp)
+				// LocalVariable(start_pc = 6, length = 40, index = 2:int __cobertura__line__number__)
+				// LocalVariable(start_pc = 6, length = 40, index = 3:int __cobertura__branch__number__)
+				LocalVariable localVariable = lvt[0];
+				if (localVariable.getStartPC() != 0) {
+					// looks suspicious so let's use this information
+					for (int j = 0; j < lvt.length && arguments.size() < method.getArgumentTypes().length; j++) {
+						localVariable = lvt[j];
+						if (localVariable.getIndex() >= startAtStackIndex) {
+							arguments.add(new MethodArgument(localVariable.getName(), localVariable.getIndex()));
+						}
+					}
+				}
+			}
 		} else {
 			// No debug info, do we have an annotation value we can rely on?
 			if (argNamesFromAnnotation != null) {
