@@ -292,9 +292,10 @@ public final class LazyClassGen {
 				hasSerialVersionUIDField = true;
 				serialVersionUIDRequiresInitialization = true;
 				// warn about what we've done?
-				if (world.getLint().calculatingSerialVersionUID.isEnabled())
+				if (world.getLint().calculatingSerialVersionUID.isEnabled()) {
 					world.getLint().calculatingSerialVersionUID.signal(new String[] { getClassName(),
 							Long.toString(calculatedSerialVersionUID) + "L" }, null, null);
+				}
 			}
 		}
 
@@ -319,7 +320,8 @@ public final class LazyClassGen {
 		ResolvedMember[] fields = type.getDeclaredFields();
 		for (int i = 0; i < fields.length; i++) {
 			ResolvedMember field = fields[i];
-			if (field.getName().equals("serialVersionUID") && field.isStatic() && field.getType().equals(ResolvedType.LONG)) {
+			if (field.getName().equals("serialVersionUID") && Modifier.isStatic(field.getModifiers())
+					&& field.getType().equals(ResolvedType.LONG)) {
 				return true;
 			}
 		}
@@ -355,23 +357,27 @@ public final class LazyClassGen {
 	 * Returns the packagename - if its the default package we return an empty string
 	 */
 	public String getPackageName() {
-		if (packageName != null)
+		if (packageName != null) {
 			return packageName;
+		}
 		String str = getInternalClassName();
 		int index = str.indexOf("<");
-		if (index != -1)
+		if (index != -1) {
 			str = str.substring(0, index); // strip off the generics guff
+		}
 		index = str.lastIndexOf("/");
-		if (index == -1)
+		if (index == -1) {
 			return "";
+		}
 		return str.substring(0, index).replace('/', '.');
 	}
 
 	public void addMethodGen(LazyMethodGen gen) {
 		// assert gen.getClassName() == super.getClassName();
 		methodGens.add(gen);
-		if (highestLineNumber < gen.highestLineNumber)
+		if (highestLineNumber < gen.highestLineNumber) {
 			highestLineNumber = gen.highestLineNumber;
+		}
 	}
 
 	public boolean removeMethodGen(LazyMethodGen gen) {
@@ -407,9 +413,10 @@ public final class LazyClassGen {
 	}
 
 	public void warnOnModifiedSerialVersionUID(ISourceLocation sourceLocation, String reason) {
-		if (isSerializable && !hasSerialVersionUIDField)
+		if (isSerializable && !hasSerialVersionUIDField) {
 			getWorld().getLint().needsSerialVersionUIDField.signal(new String[] { myType.getResolvedTypeX().getName().toString(),
 					reason }, sourceLocation, null);
+		}
 	}
 
 	public World getWorld() {
@@ -556,14 +563,16 @@ public final class LazyClassGen {
 				ResolvedType[] interfaceRTXs = myType.getDeclaredInterfaces();
 				for (int i = 0; i < interfaceRTXs.length; i++) {
 					ResolvedType typeX = interfaceRTXs[i];
-					if (typeX.isGenericType() || typeX.isParameterizedType())
+					if (typeX.isGenericType() || typeX.isParameterizedType()) {
 						needAttribute = true;
+					}
 				}
 				if (extraSuperInterfaces != null) {
 					for (int i = 0; i < extraSuperInterfaces.length; i++) {
 						ResolvedType interfaceType = extraSuperInterfaces[i];
-						if (interfaceType.isGenericType() || interfaceType.isParameterizedType())
+						if (interfaceType.isGenericType() || interfaceType.isParameterizedType()) {
 							needAttribute = true;
+						}
 					}
 				}
 			}
@@ -571,14 +580,16 @@ public final class LazyClassGen {
 			if (myType == null) {
 				ResolvedType superclassRTX = superclass;
 				if (superclassRTX != null) {
-					if (superclassRTX.isGenericType() || superclassRTX.isParameterizedType())
+					if (superclassRTX.isGenericType() || superclassRTX.isParameterizedType()) {
 						needAttribute = true;
+					}
 				}
 			} else {
 				// check the supertype
 				ResolvedType superclassRTX = getSuperClass();
-				if (superclassRTX.isGenericType() || superclassRTX.isParameterizedType())
+				if (superclassRTX.isGenericType() || superclassRTX.isParameterizedType()) {
 					needAttribute = true;
+				}
 			}
 		}
 
@@ -691,8 +702,9 @@ public final class LazyClassGen {
 			extraSuperInterfaces = x;
 		}
 		myGen.addInterface(newInterface.getRawName());
-		if (!newInterface.equals(UnresolvedType.SERIALIZABLE))
+		if (!newInterface.equals(UnresolvedType.SERIALIZABLE)) {
 			warnOnAddedInterface(newInterface.getName(), sourceLocation);
+		}
 	}
 
 	public void setSuperClass(ResolvedType newSuperclass) {
@@ -730,8 +742,9 @@ public final class LazyClassGen {
 	}
 
 	public List getChildClasses(BcelWorld world) {
-		if (classGens.isEmpty())
+		if (classGens.isEmpty()) {
 			return Collections.EMPTY_LIST;
+		}
 		List<UnwovenClassFile.ChildClass> ret = new ArrayList<UnwovenClassFile.ChildClass>();
 		for (LazyClassGen clazz : classGens) {
 			byte[] bytes = clazz.getJavaClass(world).getBytes();
@@ -751,8 +764,9 @@ public final class LazyClassGen {
 
 	public String toShortString() {
 		String s = org.aspectj.apache.bcel.classfile.Utility.accessToString(myGen.getModifiers(), true);
-		if (s != "")
+		if (s != "") {
 			s += " ";
+		}
 		s += org.aspectj.apache.bcel.classfile.Utility.classOrInterface(myGen.getModifiers());
 		s += " ";
 		s += myGen.getClassName();
@@ -774,8 +788,9 @@ public final class LazyClassGen {
 		for (Iterator iter = classGens.iterator(); iter.hasNext();) {
 			LazyClassGen element = (LazyClassGen) iter.next();
 			element.printOne(out);
-			if (iter.hasNext())
+			if (iter.hasNext()) {
 				out.println();
+			}
 		}
 	}
 
@@ -790,8 +805,9 @@ public final class LazyClassGen {
 			out.print(" implements ");
 			for (int i = 0; i < size; i++) {
 				out.print(myGen.getInterfaceNames()[i]);
-				if (i < size - 1)
+				if (i < size - 1) {
 					out.print(", ");
+				}
 			}
 		}
 		out.print(":");
@@ -809,11 +825,13 @@ public final class LazyClassGen {
 		for (Iterator iter = methodGens.iterator(); iter.hasNext();) {
 			LazyMethodGen gen = (LazyMethodGen) iter.next();
 			// we skip empty clinits
-			if (isEmptyClinit(gen))
+			if (isEmptyClinit(gen)) {
 				continue;
+			}
 			gen.print(out, (myType != null ? myType.getWeaverVersionAttribute() : WeaverVersionInfo.UNKNOWN));
-			if (iter.hasNext())
+			if (iter.hasNext()) {
 				out.println();
+			}
 		}
 		// out.println("  ATTRIBS: " + Arrays.asList(myGen.getAttributes()));
 
@@ -822,8 +840,9 @@ public final class LazyClassGen {
 
 	private boolean isEmptyClinit(LazyMethodGen gen) {
 
-		if (!gen.getName().equals("<clinit>"))
+		if (!gen.getName().equals("<clinit>")) {
 			return false;
+		}
 		// System.err.println("checking clinig: " + gen);
 		InstructionHandle start = gen.getBody().getStart();
 		while (start != null) {
@@ -850,21 +869,24 @@ public final class LazyClassGen {
 	}
 
 	public boolean isReweavable() {
-		if (myType.getWeaverState() == null)
+		if (myType.getWeaverState() == null) {
 			return true;
+		}
 		return myType.getWeaverState().isReweavable();
 	}
 
-	public Set getAspectsAffectingType() {
-		if (myType.getWeaverState() == null)
+	public Set<String> getAspectsAffectingType() {
+		if (myType.getWeaverState() == null) {
 			return null;
+		}
 		return myType.getWeaverState().getAspectsAffectingType();
 	}
 
 	public WeaverStateInfo getOrCreateWeaverStateInfo(boolean inReweavableMode) {
 		WeaverStateInfo ret = myType.getWeaverState();
-		if (ret != null)
+		if (ret != null) {
 			return ret;
+		}
 		ret = new WeaverStateInfo(inReweavableMode);
 		myType.setWeaverState(ret);
 		return ret;
@@ -877,8 +899,9 @@ public final class LazyClassGen {
 	public LazyMethodGen getStaticInitializer() {
 		for (LazyMethodGen gen : methodGens) {
 			// OPTIMIZE persist kind of member into the gen object? for clinit
-			if (gen.getName().equals("<clinit>"))
+			if (gen.getName().equals("<clinit>")) {
 				return gen;
+			}
 		}
 		LazyMethodGen clinit = new LazyMethodGen(Modifier.STATIC, Type.VOID, "<clinit>", new Type[0], NO_STRINGS, this);
 		clinit.getBody().insert(InstructionConstants.RETURN);
@@ -886,26 +909,40 @@ public final class LazyClassGen {
 		return clinit;
 	}
 
+	/**
+	 * Retrieve the ajc$preClinit method - this method captures any initialization AspectJ wants to ensure happens in a class. It is
+	 * called from the static initializer. Maintaining this separation enables overweaving to ignore join points added due to
+	 * earlier weaves. If the ajc$preClinit method cannot be found, it is created and a call to it is placed in the real static
+	 * initializer (the call is placed at the start of the static initializer).
+	 * 
+	 * @return the LazyMethodGen representing the ajc$ clinit
+	 */
 	public LazyMethodGen getAjcPreClinit() {
-		for (Iterator i = methodGens.iterator(); i.hasNext();) {
-			LazyMethodGen gen = (LazyMethodGen) i.next();
-			if (gen.getName().equals(NameMangler.AJC_PRE_CLINIT_NAME))
-				return gen;
+		if (this.isInterface()) {
+			throw new IllegalStateException();
 		}
-		LazyMethodGen ajcClinit = new LazyMethodGen(Modifier.STATIC, Type.VOID, NameMangler.AJC_PRE_CLINIT_NAME, new Type[0],
-				NO_STRINGS, this);
-		ajcClinit.getBody().insert(InstructionConstants.RETURN);
-		methodGens.add(ajcClinit);
-
-		getStaticInitializer().getBody().insert(Utility.createInvoke(getFactory(), ajcClinit));
-		return ajcClinit;
+		for (LazyMethodGen methodGen : methodGens) {
+			if (methodGen.getName().equals(NameMangler.AJC_PRE_CLINIT_NAME)) {
+				return methodGen;
+			}
+		}
+		LazyMethodGen ajcPreClinit = new LazyMethodGen(Modifier.PRIVATE | Modifier.STATIC, Type.VOID,
+				NameMangler.AJC_PRE_CLINIT_NAME, Type.NO_ARGS, NO_STRINGS, this);
+		ajcPreClinit.getBody().insert(InstructionConstants.RETURN);
+		methodGens.add(ajcPreClinit);
+		getStaticInitializer().getBody().insert(Utility.createInvoke(fact, ajcPreClinit));
+		return ajcPreClinit;
 	}
 
 	// reflective thisJoinPoint support
-	Map/* BcelShadow, Field */tjpFields = new HashMap();
+	private Map<BcelShadow, Field> tjpFields = new HashMap<BcelShadow, Field>();
+	Map<CacheKey, Field> annotationCachingFieldCache = new HashMap<CacheKey, Field>();
+	private int tjpFieldsCounter = -1; // -1 means not yet initialized
+	private int annoFieldsCounter = 0;
 	public static final ObjectType proceedingTjpType = new ObjectType("org.aspectj.lang.ProceedingJoinPoint");
 	public static final ObjectType tjpType = new ObjectType("org.aspectj.lang.JoinPoint");
 	public static final ObjectType staticTjpType = new ObjectType("org.aspectj.lang.JoinPoint$StaticPart");
+	public static final ObjectType typeForAnnotation = new ObjectType("java.lang.annotation.Annotation");
 	public static final ObjectType enclosingStaticTjpType = new ObjectType("org.aspectj.lang.JoinPoint$EnclosingStaticPart");
 	private static final ObjectType sigType = new ObjectType("org.aspectj.lang.Signature");
 	// private static final ObjectType slType =
@@ -914,9 +951,10 @@ public final class LazyClassGen {
 	private static final ObjectType classType = new ObjectType("java.lang.Class");
 
 	public Field getTjpField(BcelShadow shadow, final boolean isEnclosingJp) {
-		Field ret = (Field) tjpFields.get(shadow);
-		if (ret != null)
-			return ret;
+		Field tjpField = tjpFields.get(shadow);
+		if (tjpField != null) {
+			return tjpField;
+		}
 
 		int modifiers = Modifier.STATIC | Modifier.FINAL;
 
@@ -925,10 +963,8 @@ public final class LazyClassGen {
 		// shadow.getEnclosingMethod().getCanInline())
 
 		// If the enclosing method is around advice, we could inline the join
-		// point
-		// that has led to this shadow. If we do that then the TJP we are
-		// creating
-		// here must be PUBLIC so it is visible to the type in which the
+		// point that has led to this shadow. If we do that then the TJP we are
+		// creating here must be PUBLIC so it is visible to the type in which the
 		// advice is inlined. (PR71377)
 		LazyMethodGen encMethod = shadow.getEnclosingMethod();
 		boolean shadowIsInAroundAdvice = false;
@@ -942,18 +978,90 @@ public final class LazyClassGen {
 			modifiers |= Modifier.PRIVATE;
 		}
 		ObjectType jpType = null;
-		if (world.isTargettingAspectJRuntime12()) { // TAG:SUPPORTING12: We
-			// didn't have different
-			// staticjp types in 1.2
+		// Did not have different static joinpoint types in 1.2
+		if (world.isTargettingAspectJRuntime12()) {
 			jpType = staticTjpType;
 		} else {
 			jpType = isEnclosingJp ? enclosingStaticTjpType : staticTjpType;
 		}
-		FieldGen fGen = new FieldGen(modifiers, jpType, "ajc$tjp_" + tjpFields.size(), getConstantPool());
+		if (tjpFieldsCounter == -1) {
+			// not yet initialized, do it now
+			if (!world.isOverWeaving()) {
+				tjpFieldsCounter = 0;
+			} else {
+				List<BcelField> existingFields = getFieldGens();
+				if (existingFields == null) {
+					tjpFieldsCounter = 0;
+				} else {
+					BcelField lastField = null;
+					// OPTIMIZE: go from last to first?
+					for (BcelField field : existingFields) {
+						if (field.getName().startsWith("ajc$tjp_")) {
+							lastField = field;
+						}
+					}
+					if (lastField == null) {
+						tjpFieldsCounter = 0;
+					} else {
+						tjpFieldsCounter = Integer.parseInt(lastField.getName().substring(8)) + 1;
+						// System.out.println("tjp counter starting at " + tjpFieldsCounter);
+					}
+				}
+			}
+		}
+		FieldGen fGen = new FieldGen(modifiers, jpType, "ajc$tjp_" + tjpFieldsCounter++, getConstantPool());
 		addField(fGen);
-		ret = fGen.getField();
-		tjpFields.put(shadow, ret);
-		return ret;
+		tjpField = fGen.getField();
+		tjpFields.put(shadow, tjpField);
+		return tjpField;
+	}
+
+	/**
+	 * Create a field in the type containing the shadow where the annotation retrieved during binding can be stored - for later fast
+	 * access.
+	 * 
+	 * @param shadow the shadow at which the @annotation result is being cached
+	 * @return a field
+	 */
+	public Field getAnnotationCachingField(BcelShadow shadow, ResolvedType toType) {
+		// Multiple annotation types at a shadow. A different field would be required for each
+		CacheKey cacheKey = new CacheKey(shadow, toType);
+		Field field = annotationCachingFieldCache.get(cacheKey);
+		if (field == null) {
+			// private static Annotation ajc$anno$<nnn>
+			StringBuilder sb = new StringBuilder();
+			sb.append(NameMangler.ANNOTATION_CACHE_FIELD_NAME);
+			sb.append(annoFieldsCounter++);
+			FieldGen annotationCacheField = new FieldGen(Modifier.PRIVATE | Modifier.STATIC, typeForAnnotation, sb.toString(), cp);
+			addField(annotationCacheField);
+			field = annotationCacheField.getField();
+			annotationCachingFieldCache.put(cacheKey, field);
+		}
+		return field;
+	}
+
+	static class CacheKey {
+		private BcelShadow shadow;
+		private ResolvedType annotationType;
+
+		CacheKey(BcelShadow shadow, ResolvedType annotationType) {
+			this.shadow = shadow;
+			this.annotationType = annotationType;
+		}
+
+		@Override
+		public int hashCode() {
+			return shadow.hashCode() * 37 + annotationType.hashCode();
+		}
+
+		@Override
+		public boolean equals(Object other) {
+			if (!(other instanceof CacheKey)) {
+				return false;
+			}
+			CacheKey oCacheKey = (CacheKey) other;
+			return shadow.equals(oCacheKey.shadow) && annotationType.equals(oCacheKey.annotationType);
+		}
 	}
 
 	// FIXME ATAJ needed only for slow Aspects.aspectOf - keep or remove
@@ -977,8 +1085,9 @@ public final class LazyClassGen {
 	// }
 
 	private void addAjcInitializers() {
-		if (tjpFields.size() == 0 && !serialVersionUIDRequiresInitialization)
+		if (tjpFields.size() == 0 && !serialVersionUIDRequiresInitialization) {
 			return;
+		}
 		InstructionList il = null;
 
 		if (tjpFields.size() > 0) {
@@ -992,8 +1101,11 @@ public final class LazyClassGen {
 			il.append(InstructionFactory.PUSH(getConstantPool(), calculatedSerialVersionUID));
 			il.append(getFactory().createFieldAccess(getClassName(), "serialVersionUID", BasicType.LONG, Constants.PUTSTATIC));
 		}
-
-		getStaticInitializer().getBody().insert(il);
+		if (this.isInterface()) { // Cannot sneak stuff into another static method in an interface
+			getStaticInitializer().getBody().insert(il);
+		} else {
+			getAjcPreClinit().getBody().insert(il);
+		}
 	}
 
 	private InstructionList initializeAllTjps() {
@@ -1019,18 +1131,16 @@ public final class LazyClassGen {
 
 		list.append(InstructionFactory.createStore(factoryType, 0));
 
-		List entries = new ArrayList(tjpFields.entrySet());
-		Collections.sort(entries, new Comparator() {
-			public int compare(Object a, Object b) {
-				Map.Entry ae = (Map.Entry) a;
-				Map.Entry be = (Map.Entry) b;
-				return ((Field) ae.getValue()).getName().compareTo(((Field) be.getValue()).getName());
+		List<Map.Entry<BcelShadow, Field>> entries = new ArrayList<Map.Entry<BcelShadow, Field>>(tjpFields.entrySet());
+		Collections.sort(entries, new Comparator<Map.Entry<BcelShadow, Field>>() {
+			public int compare(Map.Entry<BcelShadow, Field> a, Map.Entry<BcelShadow, Field> b) {
+				return (a.getValue()).getName().compareTo((b.getValue()).getName());
 			}
 		});
 
-		for (Iterator i = entries.iterator(); i.hasNext();) {
-			Map.Entry entry = (Map.Entry) i.next();
-			initializeTjp(fact, list, (Field) entry.getValue(), (BcelShadow) entry.getKey());
+		for (Iterator<Map.Entry<BcelShadow, Field>> i = entries.iterator(); i.hasNext();) {
+			Map.Entry<BcelShadow, Field> entry = i.next();
+			initializeTjp(fact, list, entry.getValue(), entry.getKey());
 		}
 
 		return list;
@@ -1230,8 +1340,9 @@ public final class LazyClassGen {
 	}
 
 	protected String makeString(UnresolvedType[] types) {
-		if (types == null)
+		if (types == null) {
 			return "";
+		}
 		StringBuffer buf = new StringBuffer();
 		for (int i = 0, len = types.length; i < len; i++) {
 			buf.append(makeString(types[i]));
@@ -1241,8 +1352,9 @@ public final class LazyClassGen {
 	}
 
 	protected String makeString(String[] names) {
-		if (names == null)
+		if (names == null) {
 			return "";
+		}
 		StringBuffer buf = new StringBuffer();
 		for (int i = 0, len = names.length; i < len; i++) {
 			buf.append(names[i]);
@@ -1252,8 +1364,9 @@ public final class LazyClassGen {
 	}
 
 	public ResolvedType getType() {
-		if (myType == null)
+		if (myType == null) {
 			return null;
+		}
 		return myType.getResolvedTypeX();
 	}
 
@@ -1346,8 +1459,9 @@ public final class LazyClassGen {
 	public LazyMethodGen getLazyMethodGen(String name, String signature, boolean allowMissing) {
 		for (Iterator i = methodGens.iterator(); i.hasNext();) {
 			LazyMethodGen gen = (LazyMethodGen) i.next();
-			if (gen.getName().equals(name) && gen.getSignature().equals(signature))
+			if (gen.getName().equals(name) && gen.getSignature().equals(signature)) {
 				return gen;
+			}
 		}
 
 		if (!allowMissing) {
@@ -1365,12 +1479,14 @@ public final class LazyClassGen {
 
 		// annotations on the real thing
 		AnnotationGen agens[] = myGen.getAnnotations();
-		if (agens == null)
+		if (agens == null) {
 			return false;
+		}
 		for (int i = 0; i < agens.length; i++) {
 			AnnotationGen gen = agens[i];
-			if (t.equals(UnresolvedType.forSignature(gen.getTypeSignature())))
+			if (t.equals(UnresolvedType.forSignature(gen.getTypeSignature()))) {
 				return true;
+			}
 		}
 
 		// annotations added during this weave
@@ -1394,15 +1510,18 @@ public final class LazyClassGen {
 	// put out
 	// a warning message!
 	private boolean implementsSerializable(ResolvedType aType) {
-		if (aType.getSignature().equals(UnresolvedType.SERIALIZABLE.getSignature()))
+		if (aType.getSignature().equals(UnresolvedType.SERIALIZABLE.getSignature())) {
 			return true;
+		}
 
 		ResolvedType[] interfaces = aType.getDeclaredInterfaces();
 		for (int i = 0; i < interfaces.length; i++) {
-			if (interfaces[i].isMissing())
+			if (interfaces[i].isMissing()) {
 				continue;
-			if (implementsSerializable(interfaces[i]))
+			}
+			if (implementsSerializable(interfaces[i])) {
 				return true;
+			}
 		}
 		ResolvedType superType = aType.getSuperclass();
 		if (superType != null && !superType.isMissing()) {
@@ -1427,8 +1546,9 @@ public final class LazyClassGen {
 			if (field.getName().startsWith(prefix)) {
 				try {
 					int num = Integer.parseInt(field.getName().substring(prefix.length()));
-					if (num > highestAllocated)
+					if (num > highestAllocated) {
 						highestAllocated = num;
+					}
 				} catch (NumberFormatException nfe) {
 					// something wrong with the number on the end of that
 					// field...
