@@ -53,6 +53,7 @@ import org.aspectj.bridge.ISourceLocation;
 import org.aspectj.weaver.AjAttribute;
 import org.aspectj.weaver.AnnotationAJ;
 import org.aspectj.weaver.BCException;
+import org.aspectj.weaver.ConstantPoolReader;
 import org.aspectj.weaver.ISourceContext;
 import org.aspectj.weaver.Lint;
 import org.aspectj.weaver.Member;
@@ -65,8 +66,8 @@ import org.aspectj.weaver.AjAttribute.WeaverVersionInfo;
 public class Utility {
 
 	public static List<AjAttribute> readAjAttributes(String classname, Attribute[] as, ISourceContext context, World w,
-			AjAttribute.WeaverVersionInfo version) {
-		List<AjAttribute> l = new ArrayList<AjAttribute>();
+			AjAttribute.WeaverVersionInfo version, ConstantPoolReader dataDecompressor) {
+		List<AjAttribute> attributes = new ArrayList<AjAttribute>();
 
 		// first pass, look for version
 		List<Unknown> forSecondPass = new ArrayList<Unknown>();
@@ -78,12 +79,13 @@ public class Utility {
 				if (name.charAt(0) == 'o') { // 'o'rg.aspectj
 					if (name.startsWith(AjAttribute.AttributePrefix)) {
 						if (name.endsWith(WeaverVersionInfo.AttributeName)) {
-							version = (AjAttribute.WeaverVersionInfo) AjAttribute.read(version, name, u.getBytes(), context, w);
+							version = (AjAttribute.WeaverVersionInfo) AjAttribute.read(version, name, u.getBytes(), context, w,
+									dataDecompressor);
 							if (version.getMajorVersion() > WeaverVersionInfo.getCurrentWeaverMajorVersion()) {
 								throw new BCException(
 										"Unable to continue, this version of AspectJ supports classes built with weaver version "
 												+ WeaverVersionInfo.toCurrentVersionString() + " but the class " + classname
-												+ " is version " + version.toString());
+												+ " is version " + version.toString() + ".  Please update your AspectJ.");
 							}
 						}
 						forSecondPass.add(u);
@@ -96,12 +98,12 @@ public class Utility {
 		for (int i = forSecondPass.size() - 1; i >= 0; i--) {
 			Unknown a = forSecondPass.get(i);
 			String name = a.getName();
-			AjAttribute attr = AjAttribute.read(version, name, a.getBytes(), context, w);
+			AjAttribute attr = AjAttribute.read(version, name, a.getBytes(), context, w, dataDecompressor);
 			if (attr != null) {
-				l.add(attr);
+				attributes.add(attr);
 			}
 		}
-		return l;
+		return attributes;
 	}
 
 	/*
@@ -128,11 +130,11 @@ public class Utility {
 
 	public static Instruction createSuperInvoke(InstructionFactory fact, BcelWorld world, Member signature) {
 		short kind;
-		if (signature.isInterface()) {
+		if (Modifier.isInterface(signature.getModifiers())) {
 			throw new RuntimeException("bad");
-		} else if (signature.isPrivate() || signature.getName().equals("<init>")) {
+		} else if (Modifier.isPrivate(signature.getModifiers()) || signature.getName().equals("<init>")) {
 			throw new RuntimeException("unimplemented, possibly bad");
-		} else if (signature.isStatic()) {
+		} else if (Modifier.isStatic(signature.getModifiers())) {
 			throw new RuntimeException("bad");
 		} else {
 			kind = Constants.INVOKESPECIAL;
@@ -145,11 +147,12 @@ public class Utility {
 	// XXX don't need the world now
 	public static Instruction createInvoke(InstructionFactory fact, BcelWorld world, Member signature) {
 		short kind;
-		if (signature.isInterface()) {
+		int signatureModifiers = signature.getModifiers();
+		if (Modifier.isInterface(signatureModifiers)) {
 			kind = Constants.INVOKEINTERFACE;
-		} else if (signature.isStatic()) {
+		} else if (Modifier.isStatic(signatureModifiers)) {
 			kind = Constants.INVOKESTATIC;
-		} else if (signature.isPrivate() || signature.getName().equals("<init>")) {
+		} else if (Modifier.isPrivate(signatureModifiers) || signature.getName().equals("<init>")) {
 			kind = Constants.INVOKESPECIAL;
 		} else {
 			kind = Constants.INVOKEVIRTUAL;
@@ -165,7 +168,7 @@ public class Utility {
 
 	public static Instruction createGet(InstructionFactory fact, Member signature) {
 		short kind;
-		if (signature.isStatic()) {
+		if (Modifier.isStatic(signature.getModifiers())) {
 			kind = Constants.GETSTATIC;
 		} else {
 			kind = Constants.GETFIELD;
@@ -177,7 +180,7 @@ public class Utility {
 
 	public static Instruction createSet(InstructionFactory fact, Member signature) {
 		short kind;
-		if (signature.isStatic()) {
+		if (Modifier.isStatic(signature.getModifiers())) {
 			kind = Constants.PUTSTATIC;
 		} else {
 			kind = Constants.PUTFIELD;
@@ -698,7 +701,7 @@ public class Utility {
 
 	public static Attribute bcelAttribute(AjAttribute a, ConstantPool pool) {
 		int nameIndex = pool.addUtf8(a.getNameString());
-		byte[] bytes = a.getBytes();
+		byte[] bytes = a.getBytes(new BcelConstantPoolWriter(pool));
 		int length = bytes.length;
 
 		return new Unknown(nameIndex, length, bytes, pool);
